package java.lang;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Locale;
import java.io.Serializable;

public final class Character implements Serializable, Comparable<Character>
{
    public static final int MIN_RADIX = 2;
    public static final int MAX_RADIX = 36;
    public static final char MIN_VALUE = '\0';
    public static final char MAX_VALUE = '\uffff';
    public static final Class<Character> TYPE;
    public static final byte UNASSIGNED = 0;
    public static final byte UPPERCASE_LETTER = 1;
    public static final byte LOWERCASE_LETTER = 2;
    public static final byte TITLECASE_LETTER = 3;
    public static final byte MODIFIER_LETTER = 4;
    public static final byte OTHER_LETTER = 5;
    public static final byte NON_SPACING_MARK = 6;
    public static final byte ENCLOSING_MARK = 7;
    public static final byte COMBINING_SPACING_MARK = 8;
    public static final byte DECIMAL_DIGIT_NUMBER = 9;
    public static final byte LETTER_NUMBER = 10;
    public static final byte OTHER_NUMBER = 11;
    public static final byte SPACE_SEPARATOR = 12;
    public static final byte LINE_SEPARATOR = 13;
    public static final byte PARAGRAPH_SEPARATOR = 14;
    public static final byte CONTROL = 15;
    public static final byte FORMAT = 16;
    public static final byte PRIVATE_USE = 18;
    public static final byte SURROGATE = 19;
    public static final byte DASH_PUNCTUATION = 20;
    public static final byte START_PUNCTUATION = 21;
    public static final byte END_PUNCTUATION = 22;
    public static final byte CONNECTOR_PUNCTUATION = 23;
    public static final byte OTHER_PUNCTUATION = 24;
    public static final byte MATH_SYMBOL = 25;
    public static final byte CURRENCY_SYMBOL = 26;
    public static final byte MODIFIER_SYMBOL = 27;
    public static final byte OTHER_SYMBOL = 28;
    public static final byte INITIAL_QUOTE_PUNCTUATION = 29;
    public static final byte FINAL_QUOTE_PUNCTUATION = 30;
    static final int ERROR = -1;
    public static final byte DIRECTIONALITY_UNDEFINED = -1;
    public static final byte DIRECTIONALITY_LEFT_TO_RIGHT = 0;
    public static final byte DIRECTIONALITY_RIGHT_TO_LEFT = 1;
    public static final byte DIRECTIONALITY_RIGHT_TO_LEFT_ARABIC = 2;
    public static final byte DIRECTIONALITY_EUROPEAN_NUMBER = 3;
    public static final byte DIRECTIONALITY_EUROPEAN_NUMBER_SEPARATOR = 4;
    public static final byte DIRECTIONALITY_EUROPEAN_NUMBER_TERMINATOR = 5;
    public static final byte DIRECTIONALITY_ARABIC_NUMBER = 6;
    public static final byte DIRECTIONALITY_COMMON_NUMBER_SEPARATOR = 7;
    public static final byte DIRECTIONALITY_NONSPACING_MARK = 8;
    public static final byte DIRECTIONALITY_BOUNDARY_NEUTRAL = 9;
    public static final byte DIRECTIONALITY_PARAGRAPH_SEPARATOR = 10;
    public static final byte DIRECTIONALITY_SEGMENT_SEPARATOR = 11;
    public static final byte DIRECTIONALITY_WHITESPACE = 12;
    public static final byte DIRECTIONALITY_OTHER_NEUTRALS = 13;
    public static final byte DIRECTIONALITY_LEFT_TO_RIGHT_EMBEDDING = 14;
    public static final byte DIRECTIONALITY_LEFT_TO_RIGHT_OVERRIDE = 15;
    public static final byte DIRECTIONALITY_RIGHT_TO_LEFT_EMBEDDING = 16;
    public static final byte DIRECTIONALITY_RIGHT_TO_LEFT_OVERRIDE = 17;
    public static final byte DIRECTIONALITY_POP_DIRECTIONAL_FORMAT = 18;
    public static final char MIN_HIGH_SURROGATE = '?';
    public static final char MAX_HIGH_SURROGATE = '?';
    public static final char MIN_LOW_SURROGATE = '?';
    public static final char MAX_LOW_SURROGATE = '?';
    public static final char MIN_SURROGATE = '?';
    public static final char MAX_SURROGATE = '?';
    public static final int MIN_SUPPLEMENTARY_CODE_POINT = 65536;
    public static final int MIN_CODE_POINT = 0;
    public static final int MAX_CODE_POINT = 1114111;
    private final char value;
    private static final long serialVersionUID = 3786198910865385080L;
    public static final int SIZE = 16;
    public static final int BYTES = 2;
    
    public Character(char value) {
        this.value = value;
    }
    
    public static Character valueOf(char c) {
        if (c <= '\u007f') {
            return Character.CharacterCache.cache[c];
        }
        return new Character(c);
    }
    
    public char charValue() {
        return this.value;
    }
    
    @Override
    public int hashCode() {
        return hashCode(this.value);
    }
    
    public static int hashCode(char c) {
        return c;
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof Character && this.value == (char)o;
    }
    
    @Override
    public String toString() {
        return String.valueOf(new char[] { this.value });
    }
    
    public static String toString(char c) {
        return String.valueOf(c);
    }
    
    public static boolean isValidCodePoint(int n) {
        return n >>> 16 < 17;
    }
    
    public static boolean isBmpCodePoint(int n) {
        return n >>> 16 == 0;
    }
    
    public static boolean isSupplementaryCodePoint(int n) {
        return n >= 65536 && n < 1114112;
    }
    
    public static boolean isHighSurrogate(char c) {
        return c >= '?' && c < '?';
    }
    
    public static boolean isLowSurrogate(char c) {
        return c >= '?' && c < '';
    }
    
    public static boolean isSurrogate(char c) {
        return c >= '?' && c < '';
    }
    
    public static boolean isSurrogatePair(char c, char c2) {
        return isHighSurrogate(c) && isLowSurrogate(c2);
    }
    
    public static int charCount(int n) {
        return (n >= 65536) ? 2 : 1;
    }
    
    public static int toCodePoint(char c, char c2) {
        return (c << 10) + c2 - 56613888;
    }
    
    public static int codePointAt(CharSequence charSequence, int n) {
        int char1 = charSequence.charAt(n);
        if (isHighSurrogate((char)char1) && ++n < charSequence.length()) {
            int char2 = charSequence.charAt(n);
            if (isLowSurrogate((char)char2)) {
                return toCodePoint((char)char1, (char)char2);
            }
        }
        return char1;
    }
    
    public static int codePointAt(char[] array, int n) {
        return codePointAtImpl(array, n, array.length);
    }
    
    public static int codePointAt(char[] array, int n, int n2) {
        if (n >= n2 || n2 < 0 || n2 > array.length) {
            throw new IndexOutOfBoundsException();
        }
        return codePointAtImpl(array, n, n2);
    }
    
    static int codePointAtImpl(char[] array, int n, int n2) {
        int n3 = array[n];
        if (isHighSurrogate((char)n3) && ++n < n2) {
            int n4 = array[n];
            if (isLowSurrogate((char)n4)) {
                return toCodePoint((char)n3, (char)n4);
            }
        }
        return n3;
    }
    
    public static int codePointBefore(CharSequence charSequence, int n) {
        int char1 = charSequence.charAt(--n);
        if (isLowSurrogate((char)char1) && n > 0) {
            int char2 = charSequence.charAt(--n);
            if (isHighSurrogate((char)char2)) {
                return toCodePoint((char)char2, (char)char1);
            }
        }
        return char1;
    }
    
    public static int codePointBefore(char[] array, int n) {
        return codePointBeforeImpl(array, n, 0);
    }
    
    public static int codePointBefore(char[] array, int n, int n2) {
        if (n <= n2 || n2 < 0 || n2 >= array.length) {
            throw new IndexOutOfBoundsException();
        }
        return codePointBeforeImpl(array, n, n2);
    }
    
    static int codePointBeforeImpl(char[] array, int n, int n2) {
        int n3 = array[--n];
        if (isLowSurrogate((char)n3) && n > n2) {
            int n4 = array[--n];
            if (isHighSurrogate((char)n4)) {
                return toCodePoint((char)n4, (char)n3);
            }
        }
        return n3;
    }
    
    public static char highSurrogate(int n) {
        return (char)((n >>> 10) + 55232);
    }
    
    public static char lowSurrogate(int n) {
        return (char)((n & 0x3FF) + 56320);
    }
    
    public static int toChars(int n, char[] array, int n2) {
        if (isBmpCodePoint(n)) {
            array[n2] = (char)n;
            return 1;
        }
        if (isValidCodePoint(n)) {
            toSurrogates(n, array, n2);
            return 2;
        }
        throw new IllegalArgumentException();
    }
    
    public static char[] toChars(int n) {
        if (isBmpCodePoint(n)) {
            return new char[] { (char)n };
        }
        if (isValidCodePoint(n)) {
            char[] array = new char[2];
            toSurrogates(n, array, 0);
            return array;
        }
        throw new IllegalArgumentException();
    }
    
    static void toSurrogates(int n, char[] array, int n2) {
        array[n2 + 1] = lowSurrogate(n);
        array[n2] = highSurrogate(n);
    }
    
    public static int codePointCount(CharSequence charSequence, int i, int n) {
        int length = charSequence.length();
        if (i < 0 || n > length || i > n) {
            throw new IndexOutOfBoundsException();
        }
        int n2 = n - i;
        while (i < n) {
            if (!isHighSurrogate(charSequence.charAt(i++)) || i >= n || !isLowSurrogate(charSequence.charAt(i))) {
                continue;
            }
            --n2;
            ++i;
        }
        return n2;
    }
    
    public static int codePointCount(char[] array, int n, int n2) {
        if (n2 > array.length - n || n < 0 || n2 < 0) {
            throw new IndexOutOfBoundsException();
        }
        return codePointCountImpl(array, n, n2);
    }
    
    static int codePointCountImpl(char[] array, int i, int n) {
        int n2 = i + n;
        while (i < n2) {
            if (!isHighSurrogate(array[i++]) || i >= n2 || !isLowSurrogate(array[i])) {
                continue;
            }
            --n;
            ++i;
        }
        return n;
    }
    
    public static int offsetByCodePoints(CharSequence charSequence, int n, int n2) {
        int length = charSequence.length();
        if (n < 0 || n > length) {
            throw new IndexOutOfBoundsException();
        }
        if (n2 >= 0) {
            int n3;
            for (n3 = 0; n < length && n3 < n2; ++n3) {
                if (isHighSurrogate(charSequence.charAt(n++)) && n < length && isLowSurrogate(charSequence.charAt(n))) {
                    ++n;
                }
            }
            if (n3 < n2) {
                throw new IndexOutOfBoundsException();
            }
        }
        else {
            for (; n > 0 && n2 < 0; ++n2) {
                if (isLowSurrogate(charSequence.charAt(--n)) && n > 0 && isHighSurrogate(charSequence.charAt(n - 1))) {
                    --n;
                }
            }
            if (n2 < 0) {
                throw new IndexOutOfBoundsException();
            }
        }
        return n;
    }
    
    public static int offsetByCodePoints(char[] array, int n, int n2, int n3, int n4) {
        if (n2 > array.length - n || n < 0 || n2 < 0 || n3 < n || n3 > n + n2) {
            throw new IndexOutOfBoundsException();
        }
        return offsetByCodePointsImpl(array, n, n2, n3, n4);
    }
    
    static int offsetByCodePointsImpl(char[] array, int n, int n2, int n3, int n4) {
        if (n4 >= 0) {
            int n5;
            int n6;
            for (n5 = n + n2, n6 = 0; n3 < n5 && n6 < n4; ++n6) {
                if (isHighSurrogate(array[n3++]) && n3 < n5 && isLowSurrogate(array[n3])) {
                    ++n3;
                }
            }
            if (n6 < n4) {
                throw new IndexOutOfBoundsException();
            }
        }
        else {
            for (; n3 > n && n4 < 0; ++n4) {
                if (isLowSurrogate(array[--n3]) && n3 > n && isHighSurrogate(array[n3 - 1])) {
                    --n3;
                }
            }
            if (n4 < 0) {
                throw new IndexOutOfBoundsException();
            }
        }
        return n3;
    }
    
    public static boolean isLowerCase(char c) {
        return isLowerCase((int)c);
    }
    
    public static boolean isLowerCase(int n) {
        return getType(n) == 2 || CharacterData.of(n).isOtherLowercase(n);
    }
    
    public static boolean isUpperCase(char c) {
        return isUpperCase((int)c);
    }
    
    public static boolean isUpperCase(int n) {
        return getType(n) == 1 || CharacterData.of(n).isOtherUppercase(n);
    }
    
    public static boolean isTitleCase(char c) {
        return isTitleCase((int)c);
    }
    
    public static boolean isTitleCase(int n) {
        return getType(n) == 3;
    }
    
    public static boolean isDigit(char c) {
        return isDigit((int)c);
    }
    
    public static boolean isDigit(int n) {
        return getType(n) == 9;
    }
    
    public static boolean isDefined(char c) {
        return isDefined((int)c);
    }
    
    public static boolean isDefined(int n) {
        return getType(n) != 0;
    }
    
    public static boolean isLetter(char c) {
        return isLetter((int)c);
    }
    
    public static boolean isLetter(int n) {
        return (62 >> getType(n) & 0x1) != 0x0;
    }
    
    public static boolean isLetterOrDigit(char c) {
        return isLetterOrDigit((int)c);
    }
    
    public static boolean isLetterOrDigit(int n) {
        return (574 >> getType(n) & 0x1) != 0x0;
    }
    
    @Deprecated
    public static boolean isJavaLetter(char c) {
        return isJavaIdentifierStart(c);
    }
    
    @Deprecated
    public static boolean isJavaLetterOrDigit(char c) {
        return isJavaIdentifierPart(c);
    }
    
    public static boolean isAlphabetic(int n) {
        return (1086 >> getType(n) & 0x1) != 0x0 || CharacterData.of(n).isOtherAlphabetic(n);
    }
    
    public static boolean isIdeographic(int n) {
        return CharacterData.of(n).isIdeographic(n);
    }
    
    public static boolean isJavaIdentifierStart(char c) {
        return isJavaIdentifierStart((int)c);
    }
    
    public static boolean isJavaIdentifierStart(int n) {
        return CharacterData.of(n).isJavaIdentifierStart(n);
    }
    
    public static boolean isJavaIdentifierPart(char c) {
        return isJavaIdentifierPart((int)c);
    }
    
    public static boolean isJavaIdentifierPart(int n) {
        return CharacterData.of(n).isJavaIdentifierPart(n);
    }
    
    public static boolean isUnicodeIdentifierStart(char c) {
        return isUnicodeIdentifierStart((int)c);
    }
    
    public static boolean isUnicodeIdentifierStart(int n) {
        return CharacterData.of(n).isUnicodeIdentifierStart(n);
    }
    
    public static boolean isUnicodeIdentifierPart(char c) {
        return isUnicodeIdentifierPart((int)c);
    }
    
    public static boolean isUnicodeIdentifierPart(int n) {
        return CharacterData.of(n).isUnicodeIdentifierPart(n);
    }
    
    public static boolean isIdentifierIgnorable(char c) {
        return isIdentifierIgnorable((int)c);
    }
    
    public static boolean isIdentifierIgnorable(int n) {
        return CharacterData.of(n).isIdentifierIgnorable(n);
    }
    
    public static char toLowerCase(char c) {
        return (char)toLowerCase((int)c);
    }
    
    public static int toLowerCase(int n) {
        return CharacterData.of(n).toLowerCase(n);
    }
    
    public static char toUpperCase(char c) {
        return (char)toUpperCase((int)c);
    }
    
    public static int toUpperCase(int n) {
        return CharacterData.of(n).toUpperCase(n);
    }
    
    public static char toTitleCase(char c) {
        return (char)toTitleCase((int)c);
    }
    
    public static int toTitleCase(int n) {
        return CharacterData.of(n).toTitleCase(n);
    }
    
    public static int digit(char c, int n) {
        return digit((int)c, n);
    }
    
    public static int digit(int n, int n2) {
        return CharacterData.of(n).digit(n, n2);
    }
    
    public static int getNumericValue(char c) {
        return getNumericValue((int)c);
    }
    
    public static int getNumericValue(int n) {
        return CharacterData.of(n).getNumericValue(n);
    }
    
    @Deprecated
    public static boolean isSpace(char c) {
        return c <= ' ' && (4294981120L >> c & 0x1L) != 0x0L;
    }
    
    public static boolean isSpaceChar(char c) {
        return isSpaceChar((int)c);
    }
    
    public static boolean isSpaceChar(int n) {
        return (28672 >> getType(n) & 0x1) != 0x0;
    }
    
    public static boolean isWhitespace(char c) {
        return isWhitespace((int)c);
    }
    
    public static boolean isWhitespace(int n) {
        return CharacterData.of(n).isWhitespace(n);
    }
    
    public static boolean isISOControl(char c) {
        return isISOControl((int)c);
    }
    
    public static boolean isISOControl(int n) {
        return n <= 159 && (n >= 127 || n >>> 5 == 0);
    }
    
    public static int getType(char c) {
        return getType((int)c);
    }
    
    public static int getType(int n) {
        return CharacterData.of(n).getType(n);
    }
    
    public static char forDigit(int n, int n2) {
        if (n >= n2 || n < 0) {
            return '\0';
        }
        if (n2 < 2 || n2 > 36) {
            return '\0';
        }
        if (n < 10) {
            return (char)(48 + n);
        }
        return (char)(87 + n);
    }
    
    public static byte getDirectionality(char c) {
        return getDirectionality((int)c);
    }
    
    public static byte getDirectionality(int n) {
        return CharacterData.of(n).getDirectionality(n);
    }
    
    public static boolean isMirrored(char c) {
        return isMirrored((int)c);
    }
    
    public static boolean isMirrored(int n) {
        return CharacterData.of(n).isMirrored(n);
    }
    
    @Override
    public int compareTo(Character c) {
        return compare(this.value, c.value);
    }
    
    public static int compare(char c, char c2) {
        return c - c2;
    }
    
    static int toUpperCaseEx(int n) {
        assert !(!isValidCodePoint(n));
        return CharacterData.of(n).toUpperCaseEx(n);
    }
    
    static char[] toUpperCaseCharArray(int n) {
        assert !(!isBmpCodePoint(n));
        return CharacterData.of(n).toUpperCaseCharArray(n);
    }
    
    public static char reverseBytes(char c) {
        return (char)((c & '\uff00') >> 8 | c << 8);
    }
    
    public static String getName(int n) {
        if (!isValidCodePoint(n)) {
            throw new IllegalArgumentException();
        }
        String value = CharacterName.get(n);
        if (value != null) {
            return value;
        }
        if (getType(n) == 0) {
            return null;
        }
        Character.UnicodeBlock of = Character.UnicodeBlock.of(n);
        if (of != null) {
            return of.toString().replace('_', ' ') + " " + Integer.toHexString(n).toUpperCase(Locale.ENGLISH);
        }
        return Integer.toHexString(n).toUpperCase(Locale.ENGLISH);
    }
    
    static {
        TYPE = Class.getPrimitiveClass("char");
    }
    
    public static class Subset
    {
        private String name;
        
        protected Subset(String name) {
            if (name == null) {
                throw new NullPointerException("name");
            }
            this.name = name;
            return;
        }
        
        @Override
        public final boolean equals(Object o) {
            return this == o;
        }
        
        @Override
        public final int hashCode() {
            return super.hashCode();
        }
        
        @Override
        public final String toString() {
            return this.name;
        }
    }
    
    public static final class UnicodeBlock extends Character.Subset
    {
        private static Map<String, Character.UnicodeBlock> map;
        public static final Character.UnicodeBlock BASIC_LATIN;
        public static final Character.UnicodeBlock LATIN_1_SUPPLEMENT;
        public static final Character.UnicodeBlock LATIN_EXTENDED_A;
        public static final Character.UnicodeBlock LATIN_EXTENDED_B;
        public static final Character.UnicodeBlock IPA_EXTENSIONS;
        public static final Character.UnicodeBlock SPACING_MODIFIER_LETTERS;
        public static final Character.UnicodeBlock COMBINING_DIACRITICAL_MARKS;
        public static final Character.UnicodeBlock GREEK;
        public static final Character.UnicodeBlock CYRILLIC;
        public static final Character.UnicodeBlock ARMENIAN;
        public static final Character.UnicodeBlock HEBREW;
        public static final Character.UnicodeBlock ARABIC;
        public static final Character.UnicodeBlock DEVANAGARI;
        public static final Character.UnicodeBlock BENGALI;
        public static final Character.UnicodeBlock GURMUKHI;
        public static final Character.UnicodeBlock GUJARATI;
        public static final Character.UnicodeBlock ORIYA;
        public static final Character.UnicodeBlock TAMIL;
        public static final Character.UnicodeBlock TELUGU;
        public static final Character.UnicodeBlock KANNADA;
        public static final Character.UnicodeBlock MALAYALAM;
        public static final Character.UnicodeBlock THAI;
        public static final Character.UnicodeBlock LAO;
        public static final Character.UnicodeBlock TIBETAN;
        public static final Character.UnicodeBlock GEORGIAN;
        public static final Character.UnicodeBlock HANGUL_JAMO;
        public static final Character.UnicodeBlock LATIN_EXTENDED_ADDITIONAL;
        public static final Character.UnicodeBlock GREEK_EXTENDED;
        public static final Character.UnicodeBlock GENERAL_PUNCTUATION;
        public static final Character.UnicodeBlock SUPERSCRIPTS_AND_SUBSCRIPTS;
        public static final Character.UnicodeBlock CURRENCY_SYMBOLS;
        public static final Character.UnicodeBlock COMBINING_MARKS_FOR_SYMBOLS;
        public static final Character.UnicodeBlock LETTERLIKE_SYMBOLS;
        public static final Character.UnicodeBlock NUMBER_FORMS;
        public static final Character.UnicodeBlock ARROWS;
        public static final Character.UnicodeBlock MATHEMATICAL_OPERATORS;
        public static final Character.UnicodeBlock MISCELLANEOUS_TECHNICAL;
        public static final Character.UnicodeBlock CONTROL_PICTURES;
        public static final Character.UnicodeBlock OPTICAL_CHARACTER_RECOGNITION;
        public static final Character.UnicodeBlock ENCLOSED_ALPHANUMERICS;
        public static final Character.UnicodeBlock BOX_DRAWING;
        public static final Character.UnicodeBlock BLOCK_ELEMENTS;
        public static final Character.UnicodeBlock GEOMETRIC_SHAPES;
        public static final Character.UnicodeBlock MISCELLANEOUS_SYMBOLS;
        public static final Character.UnicodeBlock DINGBATS;
        public static final Character.UnicodeBlock CJK_SYMBOLS_AND_PUNCTUATION;
        public static final Character.UnicodeBlock HIRAGANA;
        public static final Character.UnicodeBlock KATAKANA;
        public static final Character.UnicodeBlock BOPOMOFO;
        public static final Character.UnicodeBlock HANGUL_COMPATIBILITY_JAMO;
        public static final Character.UnicodeBlock KANBUN;
        public static final Character.UnicodeBlock ENCLOSED_CJK_LETTERS_AND_MONTHS;
        public static final Character.UnicodeBlock CJK_COMPATIBILITY;
        public static final Character.UnicodeBlock CJK_UNIFIED_IDEOGRAPHS;
        public static final Character.UnicodeBlock HANGUL_SYLLABLES;
        public static final Character.UnicodeBlock PRIVATE_USE_AREA;
        public static final Character.UnicodeBlock CJK_COMPATIBILITY_IDEOGRAPHS;
        public static final Character.UnicodeBlock ALPHABETIC_PRESENTATION_FORMS;
        public static final Character.UnicodeBlock ARABIC_PRESENTATION_FORMS_A;
        public static final Character.UnicodeBlock COMBINING_HALF_MARKS;
        public static final Character.UnicodeBlock CJK_COMPATIBILITY_FORMS;
        public static final Character.UnicodeBlock SMALL_FORM_VARIANTS;
        public static final Character.UnicodeBlock ARABIC_PRESENTATION_FORMS_B;
        public static final Character.UnicodeBlock HALFWIDTH_AND_FULLWIDTH_FORMS;
        public static final Character.UnicodeBlock SPECIALS;
        @Deprecated
        public static final Character.UnicodeBlock SURROGATES_AREA;
        public static final Character.UnicodeBlock SYRIAC;
        public static final Character.UnicodeBlock THAANA;
        public static final Character.UnicodeBlock SINHALA;
        public static final Character.UnicodeBlock MYANMAR;
        public static final Character.UnicodeBlock ETHIOPIC;
        public static final Character.UnicodeBlock CHEROKEE;
        public static final Character.UnicodeBlock UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS;
        public static final Character.UnicodeBlock OGHAM;
        public static final Character.UnicodeBlock RUNIC;
        public static final Character.UnicodeBlock KHMER;
        public static final Character.UnicodeBlock MONGOLIAN;
        public static final Character.UnicodeBlock BRAILLE_PATTERNS;
        public static final Character.UnicodeBlock CJK_RADICALS_SUPPLEMENT;
        public static final Character.UnicodeBlock KANGXI_RADICALS;
        public static final Character.UnicodeBlock IDEOGRAPHIC_DESCRIPTION_CHARACTERS;
        public static final Character.UnicodeBlock BOPOMOFO_EXTENDED;
        public static final Character.UnicodeBlock CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A;
        public static final Character.UnicodeBlock YI_SYLLABLES;
        public static final Character.UnicodeBlock YI_RADICALS;
        public static final Character.UnicodeBlock CYRILLIC_SUPPLEMENTARY;
        public static final Character.UnicodeBlock TAGALOG;
        public static final Character.UnicodeBlock HANUNOO;
        public static final Character.UnicodeBlock BUHID;
        public static final Character.UnicodeBlock TAGBANWA;
        public static final Character.UnicodeBlock LIMBU;
        public static final Character.UnicodeBlock TAI_LE;
        public static final Character.UnicodeBlock KHMER_SYMBOLS;
        public static final Character.UnicodeBlock PHONETIC_EXTENSIONS;
        public static final Character.UnicodeBlock MISCELLANEOUS_MATHEMATICAL_SYMBOLS_A;
        public static final Character.UnicodeBlock SUPPLEMENTAL_ARROWS_A;
        public static final Character.UnicodeBlock SUPPLEMENTAL_ARROWS_B;
        public static final Character.UnicodeBlock MISCELLANEOUS_MATHEMATICAL_SYMBOLS_B;
        public static final Character.UnicodeBlock SUPPLEMENTAL_MATHEMATICAL_OPERATORS;
        public static final Character.UnicodeBlock MISCELLANEOUS_SYMBOLS_AND_ARROWS;
        public static final Character.UnicodeBlock KATAKANA_PHONETIC_EXTENSIONS;
        public static final Character.UnicodeBlock YIJING_HEXAGRAM_SYMBOLS;
        public static final Character.UnicodeBlock VARIATION_SELECTORS;
        public static final Character.UnicodeBlock LINEAR_B_SYLLABARY;
        public static final Character.UnicodeBlock LINEAR_B_IDEOGRAMS;
        public static final Character.UnicodeBlock AEGEAN_NUMBERS;
        public static final Character.UnicodeBlock OLD_ITALIC;
        public static final Character.UnicodeBlock GOTHIC;
        public static final Character.UnicodeBlock UGARITIC;
        public static final Character.UnicodeBlock DESERET;
        public static final Character.UnicodeBlock SHAVIAN;
        public static final Character.UnicodeBlock OSMANYA;
        public static final Character.UnicodeBlock CYPRIOT_SYLLABARY;
        public static final Character.UnicodeBlock BYZANTINE_MUSICAL_SYMBOLS;
        public static final Character.UnicodeBlock MUSICAL_SYMBOLS;
        public static final Character.UnicodeBlock TAI_XUAN_JING_SYMBOLS;
        public static final Character.UnicodeBlock MATHEMATICAL_ALPHANUMERIC_SYMBOLS;
        public static final Character.UnicodeBlock CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B;
        public static final Character.UnicodeBlock CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT;
        public static final Character.UnicodeBlock TAGS;
        public static final Character.UnicodeBlock VARIATION_SELECTORS_SUPPLEMENT;
        public static final Character.UnicodeBlock SUPPLEMENTARY_PRIVATE_USE_AREA_A;
        public static final Character.UnicodeBlock SUPPLEMENTARY_PRIVATE_USE_AREA_B;
        public static final Character.UnicodeBlock HIGH_SURROGATES;
        public static final Character.UnicodeBlock HIGH_PRIVATE_USE_SURROGATES;
        public static final Character.UnicodeBlock LOW_SURROGATES;
        public static final Character.UnicodeBlock ARABIC_SUPPLEMENT;
        public static final Character.UnicodeBlock NKO;
        public static final Character.UnicodeBlock SAMARITAN;
        public static final Character.UnicodeBlock MANDAIC;
        public static final Character.UnicodeBlock ETHIOPIC_SUPPLEMENT;
        public static final Character.UnicodeBlock UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS_EXTENDED;
        public static final Character.UnicodeBlock NEW_TAI_LUE;
        public static final Character.UnicodeBlock BUGINESE;
        public static final Character.UnicodeBlock TAI_THAM;
        public static final Character.UnicodeBlock BALINESE;
        public static final Character.UnicodeBlock SUNDANESE;
        public static final Character.UnicodeBlock BATAK;
        public static final Character.UnicodeBlock LEPCHA;
        public static final Character.UnicodeBlock OL_CHIKI;
        public static final Character.UnicodeBlock VEDIC_EXTENSIONS;
        public static final Character.UnicodeBlock PHONETIC_EXTENSIONS_SUPPLEMENT;
        public static final Character.UnicodeBlock COMBINING_DIACRITICAL_MARKS_SUPPLEMENT;
        public static final Character.UnicodeBlock GLAGOLITIC;
        public static final Character.UnicodeBlock LATIN_EXTENDED_C;
        public static final Character.UnicodeBlock COPTIC;
        public static final Character.UnicodeBlock GEORGIAN_SUPPLEMENT;
        public static final Character.UnicodeBlock TIFINAGH;
        public static final Character.UnicodeBlock ETHIOPIC_EXTENDED;
        public static final Character.UnicodeBlock CYRILLIC_EXTENDED_A;
        public static final Character.UnicodeBlock SUPPLEMENTAL_PUNCTUATION;
        public static final Character.UnicodeBlock CJK_STROKES;
        public static final Character.UnicodeBlock LISU;
        public static final Character.UnicodeBlock VAI;
        public static final Character.UnicodeBlock CYRILLIC_EXTENDED_B;
        public static final Character.UnicodeBlock BAMUM;
        public static final Character.UnicodeBlock MODIFIER_TONE_LETTERS;
        public static final Character.UnicodeBlock LATIN_EXTENDED_D;
        public static final Character.UnicodeBlock SYLOTI_NAGRI;
        public static final Character.UnicodeBlock COMMON_INDIC_NUMBER_FORMS;
        public static final Character.UnicodeBlock PHAGS_PA;
        public static final Character.UnicodeBlock SAURASHTRA;
        public static final Character.UnicodeBlock DEVANAGARI_EXTENDED;
        public static final Character.UnicodeBlock KAYAH_LI;
        public static final Character.UnicodeBlock REJANG;
        public static final Character.UnicodeBlock HANGUL_JAMO_EXTENDED_A;
        public static final Character.UnicodeBlock JAVANESE;
        public static final Character.UnicodeBlock CHAM;
        public static final Character.UnicodeBlock MYANMAR_EXTENDED_A;
        public static final Character.UnicodeBlock TAI_VIET;
        public static final Character.UnicodeBlock ETHIOPIC_EXTENDED_A;
        public static final Character.UnicodeBlock MEETEI_MAYEK;
        public static final Character.UnicodeBlock HANGUL_JAMO_EXTENDED_B;
        public static final Character.UnicodeBlock VERTICAL_FORMS;
        public static final Character.UnicodeBlock ANCIENT_GREEK_NUMBERS;
        public static final Character.UnicodeBlock ANCIENT_SYMBOLS;
        public static final Character.UnicodeBlock PHAISTOS_DISC;
        public static final Character.UnicodeBlock LYCIAN;
        public static final Character.UnicodeBlock CARIAN;
        public static final Character.UnicodeBlock OLD_PERSIAN;
        public static final Character.UnicodeBlock IMPERIAL_ARAMAIC;
        public static final Character.UnicodeBlock PHOENICIAN;
        public static final Character.UnicodeBlock LYDIAN;
        public static final Character.UnicodeBlock KHAROSHTHI;
        public static final Character.UnicodeBlock OLD_SOUTH_ARABIAN;
        public static final Character.UnicodeBlock AVESTAN;
        public static final Character.UnicodeBlock INSCRIPTIONAL_PARTHIAN;
        public static final Character.UnicodeBlock INSCRIPTIONAL_PAHLAVI;
        public static final Character.UnicodeBlock OLD_TURKIC;
        public static final Character.UnicodeBlock RUMI_NUMERAL_SYMBOLS;
        public static final Character.UnicodeBlock BRAHMI;
        public static final Character.UnicodeBlock KAITHI;
        public static final Character.UnicodeBlock CUNEIFORM;
        public static final Character.UnicodeBlock CUNEIFORM_NUMBERS_AND_PUNCTUATION;
        public static final Character.UnicodeBlock EGYPTIAN_HIEROGLYPHS;
        public static final Character.UnicodeBlock BAMUM_SUPPLEMENT;
        public static final Character.UnicodeBlock KANA_SUPPLEMENT;
        public static final Character.UnicodeBlock ANCIENT_GREEK_MUSICAL_NOTATION;
        public static final Character.UnicodeBlock COUNTING_ROD_NUMERALS;
        public static final Character.UnicodeBlock MAHJONG_TILES;
        public static final Character.UnicodeBlock DOMINO_TILES;
        public static final Character.UnicodeBlock PLAYING_CARDS;
        public static final Character.UnicodeBlock ENCLOSED_ALPHANUMERIC_SUPPLEMENT;
        public static final Character.UnicodeBlock ENCLOSED_IDEOGRAPHIC_SUPPLEMENT;
        public static final Character.UnicodeBlock MISCELLANEOUS_SYMBOLS_AND_PICTOGRAPHS;
        public static final Character.UnicodeBlock EMOTICONS;
        public static final Character.UnicodeBlock TRANSPORT_AND_MAP_SYMBOLS;
        public static final Character.UnicodeBlock ALCHEMICAL_SYMBOLS;
        public static final Character.UnicodeBlock CJK_UNIFIED_IDEOGRAPHS_EXTENSION_C;
        public static final Character.UnicodeBlock CJK_UNIFIED_IDEOGRAPHS_EXTENSION_D;
        public static final Character.UnicodeBlock ARABIC_EXTENDED_A;
        public static final Character.UnicodeBlock SUNDANESE_SUPPLEMENT;
        public static final Character.UnicodeBlock MEETEI_MAYEK_EXTENSIONS;
        public static final Character.UnicodeBlock MEROITIC_HIEROGLYPHS;
        public static final Character.UnicodeBlock MEROITIC_CURSIVE;
        public static final Character.UnicodeBlock SORA_SOMPENG;
        public static final Character.UnicodeBlock CHAKMA;
        public static final Character.UnicodeBlock SHARADA;
        public static final Character.UnicodeBlock TAKRI;
        public static final Character.UnicodeBlock MIAO;
        public static final Character.UnicodeBlock ARABIC_MATHEMATICAL_ALPHABETIC_SYMBOLS;
        private static final int[] blockStarts;
        private static final Character.UnicodeBlock[] blocks;
        
        private UnicodeBlock(String s) {
            super(s);
            Character.UnicodeBlock.map.put(s, this);
        }
        
        private UnicodeBlock(String s, String s2) {
            this(s);
            Character.UnicodeBlock.map.put(s2, this);
        }
        
        private UnicodeBlock(String s, String... array) {
            this(s);
            for (int length = array.length, i = 0; i < length; ++i) {
                Character.UnicodeBlock.map.put(array[i], this);
            }
        }
        
        public static Character.UnicodeBlock of(char c) {
            return of((int)c);
        }
        
        public static Character.UnicodeBlock of(int n) {
            if (!Character.isValidCodePoint(n)) {
                throw new IllegalArgumentException();
            }
            int n2 = 0;
            int length = Character.UnicodeBlock.blockStarts.length;
            int n3 = length / 2;
            while (length - n2 > 1) {
                if (n >= Character.UnicodeBlock.blockStarts[n3]) {
                    n2 = n3;
                }
                else {
                    length = n3;
                }
                n3 = (length + n2) / 2;
            }
            return Character.UnicodeBlock.blocks[n3];
        }
        
        public static final Character.UnicodeBlock forName(String s) {
            Character.UnicodeBlock unicodeBlock = (Character.UnicodeBlock)Character.UnicodeBlock.map.get(s.toUpperCase(Locale.US));
            if (unicodeBlock == null) {
                throw new IllegalArgumentException();
            }
            return unicodeBlock;
        }
        
        static {
            Character.UnicodeBlock.map = new HashMap(256);
            BASIC_LATIN = new Character.UnicodeBlock("BASIC_LATIN", new String[] { "BASIC LATIN", "BASICLATIN" });
            LATIN_1_SUPPLEMENT = new Character.UnicodeBlock("LATIN_1_SUPPLEMENT", new String[] { "LATIN-1 SUPPLEMENT", "LATIN-1SUPPLEMENT" });
            LATIN_EXTENDED_A = new Character.UnicodeBlock("LATIN_EXTENDED_A", new String[] { "LATIN EXTENDED-A", "LATINEXTENDED-A" });
            LATIN_EXTENDED_B = new Character.UnicodeBlock("LATIN_EXTENDED_B", new String[] { "LATIN EXTENDED-B", "LATINEXTENDED-B" });
            IPA_EXTENSIONS = new Character.UnicodeBlock("IPA_EXTENSIONS", new String[] { "IPA EXTENSIONS", "IPAEXTENSIONS" });
            SPACING_MODIFIER_LETTERS = new Character.UnicodeBlock("SPACING_MODIFIER_LETTERS", new String[] { "SPACING MODIFIER LETTERS", "SPACINGMODIFIERLETTERS" });
            COMBINING_DIACRITICAL_MARKS = new Character.UnicodeBlock("COMBINING_DIACRITICAL_MARKS", new String[] { "COMBINING DIACRITICAL MARKS", "COMBININGDIACRITICALMARKS" });
            GREEK = new Character.UnicodeBlock("GREEK", new String[] { "GREEK AND COPTIC", "GREEKANDCOPTIC" });
            CYRILLIC = new Character.UnicodeBlock("CYRILLIC");
            ARMENIAN = new Character.UnicodeBlock("ARMENIAN");
            HEBREW = new Character.UnicodeBlock("HEBREW");
            ARABIC = new Character.UnicodeBlock("ARABIC");
            DEVANAGARI = new Character.UnicodeBlock("DEVANAGARI");
            BENGALI = new Character.UnicodeBlock("BENGALI");
            GURMUKHI = new Character.UnicodeBlock("GURMUKHI");
            GUJARATI = new Character.UnicodeBlock("GUJARATI");
            ORIYA = new Character.UnicodeBlock("ORIYA");
            TAMIL = new Character.UnicodeBlock("TAMIL");
            TELUGU = new Character.UnicodeBlock("TELUGU");
            KANNADA = new Character.UnicodeBlock("KANNADA");
            MALAYALAM = new Character.UnicodeBlock("MALAYALAM");
            THAI = new Character.UnicodeBlock("THAI");
            LAO = new Character.UnicodeBlock("LAO");
            TIBETAN = new Character.UnicodeBlock("TIBETAN");
            GEORGIAN = new Character.UnicodeBlock("GEORGIAN");
            HANGUL_JAMO = new Character.UnicodeBlock("HANGUL_JAMO", new String[] { "HANGUL JAMO", "HANGULJAMO" });
            LATIN_EXTENDED_ADDITIONAL = new Character.UnicodeBlock("LATIN_EXTENDED_ADDITIONAL", new String[] { "LATIN EXTENDED ADDITIONAL", "LATINEXTENDEDADDITIONAL" });
            GREEK_EXTENDED = new Character.UnicodeBlock("GREEK_EXTENDED", new String[] { "GREEK EXTENDED", "GREEKEXTENDED" });
            GENERAL_PUNCTUATION = new Character.UnicodeBlock("GENERAL_PUNCTUATION", new String[] { "GENERAL PUNCTUATION", "GENERALPUNCTUATION" });
            SUPERSCRIPTS_AND_SUBSCRIPTS = new Character.UnicodeBlock("SUPERSCRIPTS_AND_SUBSCRIPTS", new String[] { "SUPERSCRIPTS AND SUBSCRIPTS", "SUPERSCRIPTSANDSUBSCRIPTS" });
            CURRENCY_SYMBOLS = new Character.UnicodeBlock("CURRENCY_SYMBOLS", new String[] { "CURRENCY SYMBOLS", "CURRENCYSYMBOLS" });
            COMBINING_MARKS_FOR_SYMBOLS = new Character.UnicodeBlock("COMBINING_MARKS_FOR_SYMBOLS", new String[] { "COMBINING DIACRITICAL MARKS FOR SYMBOLS", "COMBININGDIACRITICALMARKSFORSYMBOLS", "COMBINING MARKS FOR SYMBOLS", "COMBININGMARKSFORSYMBOLS" });
            LETTERLIKE_SYMBOLS = new Character.UnicodeBlock("LETTERLIKE_SYMBOLS", new String[] { "LETTERLIKE SYMBOLS", "LETTERLIKESYMBOLS" });
            NUMBER_FORMS = new Character.UnicodeBlock("NUMBER_FORMS", new String[] { "NUMBER FORMS", "NUMBERFORMS" });
            ARROWS = new Character.UnicodeBlock("ARROWS");
            MATHEMATICAL_OPERATORS = new Character.UnicodeBlock("MATHEMATICAL_OPERATORS", new String[] { "MATHEMATICAL OPERATORS", "MATHEMATICALOPERATORS" });
            MISCELLANEOUS_TECHNICAL = new Character.UnicodeBlock("MISCELLANEOUS_TECHNICAL", new String[] { "MISCELLANEOUS TECHNICAL", "MISCELLANEOUSTECHNICAL" });
            CONTROL_PICTURES = new Character.UnicodeBlock("CONTROL_PICTURES", new String[] { "CONTROL PICTURES", "CONTROLPICTURES" });
            OPTICAL_CHARACTER_RECOGNITION = new Character.UnicodeBlock("OPTICAL_CHARACTER_RECOGNITION", new String[] { "OPTICAL CHARACTER RECOGNITION", "OPTICALCHARACTERRECOGNITION" });
            ENCLOSED_ALPHANUMERICS = new Character.UnicodeBlock("ENCLOSED_ALPHANUMERICS", new String[] { "ENCLOSED ALPHANUMERICS", "ENCLOSEDALPHANUMERICS" });
            BOX_DRAWING = new Character.UnicodeBlock("BOX_DRAWING", new String[] { "BOX DRAWING", "BOXDRAWING" });
            BLOCK_ELEMENTS = new Character.UnicodeBlock("BLOCK_ELEMENTS", new String[] { "BLOCK ELEMENTS", "BLOCKELEMENTS" });
            GEOMETRIC_SHAPES = new Character.UnicodeBlock("GEOMETRIC_SHAPES", new String[] { "GEOMETRIC SHAPES", "GEOMETRICSHAPES" });
            MISCELLANEOUS_SYMBOLS = new Character.UnicodeBlock("MISCELLANEOUS_SYMBOLS", new String[] { "MISCELLANEOUS SYMBOLS", "MISCELLANEOUSSYMBOLS" });
            DINGBATS = new Character.UnicodeBlock("DINGBATS");
            CJK_SYMBOLS_AND_PUNCTUATION = new Character.UnicodeBlock("CJK_SYMBOLS_AND_PUNCTUATION", new String[] { "CJK SYMBOLS AND PUNCTUATION", "CJKSYMBOLSANDPUNCTUATION" });
            HIRAGANA = new Character.UnicodeBlock("HIRAGANA");
            KATAKANA = new Character.UnicodeBlock("KATAKANA");
            BOPOMOFO = new Character.UnicodeBlock("BOPOMOFO");
            HANGUL_COMPATIBILITY_JAMO = new Character.UnicodeBlock("HANGUL_COMPATIBILITY_JAMO", new String[] { "HANGUL COMPATIBILITY JAMO", "HANGULCOMPATIBILITYJAMO" });
            KANBUN = new Character.UnicodeBlock("KANBUN");
            ENCLOSED_CJK_LETTERS_AND_MONTHS = new Character.UnicodeBlock("ENCLOSED_CJK_LETTERS_AND_MONTHS", new String[] { "ENCLOSED CJK LETTERS AND MONTHS", "ENCLOSEDCJKLETTERSANDMONTHS" });
            CJK_COMPATIBILITY = new Character.UnicodeBlock("CJK_COMPATIBILITY", new String[] { "CJK COMPATIBILITY", "CJKCOMPATIBILITY" });
            CJK_UNIFIED_IDEOGRAPHS = new Character.UnicodeBlock("CJK_UNIFIED_IDEOGRAPHS", new String[] { "CJK UNIFIED IDEOGRAPHS", "CJKUNIFIEDIDEOGRAPHS" });
            HANGUL_SYLLABLES = new Character.UnicodeBlock("HANGUL_SYLLABLES", new String[] { "HANGUL SYLLABLES", "HANGULSYLLABLES" });
            PRIVATE_USE_AREA = new Character.UnicodeBlock("PRIVATE_USE_AREA", new String[] { "PRIVATE USE AREA", "PRIVATEUSEAREA" });
            CJK_COMPATIBILITY_IDEOGRAPHS = new Character.UnicodeBlock("CJK_COMPATIBILITY_IDEOGRAPHS", new String[] { "CJK COMPATIBILITY IDEOGRAPHS", "CJKCOMPATIBILITYIDEOGRAPHS" });
            ALPHABETIC_PRESENTATION_FORMS = new Character.UnicodeBlock("ALPHABETIC_PRESENTATION_FORMS", new String[] { "ALPHABETIC PRESENTATION FORMS", "ALPHABETICPRESENTATIONFORMS" });
            ARABIC_PRESENTATION_FORMS_A = new Character.UnicodeBlock("ARABIC_PRESENTATION_FORMS_A", new String[] { "ARABIC PRESENTATION FORMS-A", "ARABICPRESENTATIONFORMS-A" });
            COMBINING_HALF_MARKS = new Character.UnicodeBlock("COMBINING_HALF_MARKS", new String[] { "COMBINING HALF MARKS", "COMBININGHALFMARKS" });
            CJK_COMPATIBILITY_FORMS = new Character.UnicodeBlock("CJK_COMPATIBILITY_FORMS", new String[] { "CJK COMPATIBILITY FORMS", "CJKCOMPATIBILITYFORMS" });
            SMALL_FORM_VARIANTS = new Character.UnicodeBlock("SMALL_FORM_VARIANTS", new String[] { "SMALL FORM VARIANTS", "SMALLFORMVARIANTS" });
            ARABIC_PRESENTATION_FORMS_B = new Character.UnicodeBlock("ARABIC_PRESENTATION_FORMS_B", new String[] { "ARABIC PRESENTATION FORMS-B", "ARABICPRESENTATIONFORMS-B" });
            HALFWIDTH_AND_FULLWIDTH_FORMS = new Character.UnicodeBlock("HALFWIDTH_AND_FULLWIDTH_FORMS", new String[] { "HALFWIDTH AND FULLWIDTH FORMS", "HALFWIDTHANDFULLWIDTHFORMS" });
            SPECIALS = new Character.UnicodeBlock("SPECIALS");
            SURROGATES_AREA = new Character.UnicodeBlock("SURROGATES_AREA");
            SYRIAC = new Character.UnicodeBlock("SYRIAC");
            THAANA = new Character.UnicodeBlock("THAANA");
            SINHALA = new Character.UnicodeBlock("SINHALA");
            MYANMAR = new Character.UnicodeBlock("MYANMAR");
            ETHIOPIC = new Character.UnicodeBlock("ETHIOPIC");
            CHEROKEE = new Character.UnicodeBlock("CHEROKEE");
            UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS = new Character.UnicodeBlock("UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS", new String[] { "UNIFIED CANADIAN ABORIGINAL SYLLABICS", "UNIFIEDCANADIANABORIGINALSYLLABICS" });
            OGHAM = new Character.UnicodeBlock("OGHAM");
            RUNIC = new Character.UnicodeBlock("RUNIC");
            KHMER = new Character.UnicodeBlock("KHMER");
            MONGOLIAN = new Character.UnicodeBlock("MONGOLIAN");
            BRAILLE_PATTERNS = new Character.UnicodeBlock("BRAILLE_PATTERNS", new String[] { "BRAILLE PATTERNS", "BRAILLEPATTERNS" });
            CJK_RADICALS_SUPPLEMENT = new Character.UnicodeBlock("CJK_RADICALS_SUPPLEMENT", new String[] { "CJK RADICALS SUPPLEMENT", "CJKRADICALSSUPPLEMENT" });
            KANGXI_RADICALS = new Character.UnicodeBlock("KANGXI_RADICALS", new String[] { "KANGXI RADICALS", "KANGXIRADICALS" });
            IDEOGRAPHIC_DESCRIPTION_CHARACTERS = new Character.UnicodeBlock("IDEOGRAPHIC_DESCRIPTION_CHARACTERS", new String[] { "IDEOGRAPHIC DESCRIPTION CHARACTERS", "IDEOGRAPHICDESCRIPTIONCHARACTERS" });
            BOPOMOFO_EXTENDED = new Character.UnicodeBlock("BOPOMOFO_EXTENDED", new String[] { "BOPOMOFO EXTENDED", "BOPOMOFOEXTENDED" });
            CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A = new Character.UnicodeBlock("CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A", new String[] { "CJK UNIFIED IDEOGRAPHS EXTENSION A", "CJKUNIFIEDIDEOGRAPHSEXTENSIONA" });
            YI_SYLLABLES = new Character.UnicodeBlock("YI_SYLLABLES", new String[] { "YI SYLLABLES", "YISYLLABLES" });
            YI_RADICALS = new Character.UnicodeBlock("YI_RADICALS", new String[] { "YI RADICALS", "YIRADICALS" });
            CYRILLIC_SUPPLEMENTARY = new Character.UnicodeBlock("CYRILLIC_SUPPLEMENTARY", new String[] { "CYRILLIC SUPPLEMENTARY", "CYRILLICSUPPLEMENTARY", "CYRILLIC SUPPLEMENT", "CYRILLICSUPPLEMENT" });
            TAGALOG = new Character.UnicodeBlock("TAGALOG");
            HANUNOO = new Character.UnicodeBlock("HANUNOO");
            BUHID = new Character.UnicodeBlock("BUHID");
            TAGBANWA = new Character.UnicodeBlock("TAGBANWA");
            LIMBU = new Character.UnicodeBlock("LIMBU");
            TAI_LE = new Character.UnicodeBlock("TAI_LE", new String[] { "TAI LE", "TAILE" });
            KHMER_SYMBOLS = new Character.UnicodeBlock("KHMER_SYMBOLS", new String[] { "KHMER SYMBOLS", "KHMERSYMBOLS" });
            PHONETIC_EXTENSIONS = new Character.UnicodeBlock("PHONETIC_EXTENSIONS", new String[] { "PHONETIC EXTENSIONS", "PHONETICEXTENSIONS" });
            MISCELLANEOUS_MATHEMATICAL_SYMBOLS_A = new Character.UnicodeBlock("MISCELLANEOUS_MATHEMATICAL_SYMBOLS_A", new String[] { "MISCELLANEOUS MATHEMATICAL SYMBOLS-A", "MISCELLANEOUSMATHEMATICALSYMBOLS-A" });
            SUPPLEMENTAL_ARROWS_A = new Character.UnicodeBlock("SUPPLEMENTAL_ARROWS_A", new String[] { "SUPPLEMENTAL ARROWS-A", "SUPPLEMENTALARROWS-A" });
            SUPPLEMENTAL_ARROWS_B = new Character.UnicodeBlock("SUPPLEMENTAL_ARROWS_B", new String[] { "SUPPLEMENTAL ARROWS-B", "SUPPLEMENTALARROWS-B" });
            MISCELLANEOUS_MATHEMATICAL_SYMBOLS_B = new Character.UnicodeBlock("MISCELLANEOUS_MATHEMATICAL_SYMBOLS_B", new String[] { "MISCELLANEOUS MATHEMATICAL SYMBOLS-B", "MISCELLANEOUSMATHEMATICALSYMBOLS-B" });
            SUPPLEMENTAL_MATHEMATICAL_OPERATORS = new Character.UnicodeBlock("SUPPLEMENTAL_MATHEMATICAL_OPERATORS", new String[] { "SUPPLEMENTAL MATHEMATICAL OPERATORS", "SUPPLEMENTALMATHEMATICALOPERATORS" });
            MISCELLANEOUS_SYMBOLS_AND_ARROWS = new Character.UnicodeBlock("MISCELLANEOUS_SYMBOLS_AND_ARROWS", new String[] { "MISCELLANEOUS SYMBOLS AND ARROWS", "MISCELLANEOUSSYMBOLSANDARROWS" });
            KATAKANA_PHONETIC_EXTENSIONS = new Character.UnicodeBlock("KATAKANA_PHONETIC_EXTENSIONS", new String[] { "KATAKANA PHONETIC EXTENSIONS", "KATAKANAPHONETICEXTENSIONS" });
            YIJING_HEXAGRAM_SYMBOLS = new Character.UnicodeBlock("YIJING_HEXAGRAM_SYMBOLS", new String[] { "YIJING HEXAGRAM SYMBOLS", "YIJINGHEXAGRAMSYMBOLS" });
            VARIATION_SELECTORS = new Character.UnicodeBlock("VARIATION_SELECTORS", new String[] { "VARIATION SELECTORS", "VARIATIONSELECTORS" });
            LINEAR_B_SYLLABARY = new Character.UnicodeBlock("LINEAR_B_SYLLABARY", new String[] { "LINEAR B SYLLABARY", "LINEARBSYLLABARY" });
            LINEAR_B_IDEOGRAMS = new Character.UnicodeBlock("LINEAR_B_IDEOGRAMS", new String[] { "LINEAR B IDEOGRAMS", "LINEARBIDEOGRAMS" });
            AEGEAN_NUMBERS = new Character.UnicodeBlock("AEGEAN_NUMBERS", new String[] { "AEGEAN NUMBERS", "AEGEANNUMBERS" });
            OLD_ITALIC = new Character.UnicodeBlock("OLD_ITALIC", new String[] { "OLD ITALIC", "OLDITALIC" });
            GOTHIC = new Character.UnicodeBlock("GOTHIC");
            UGARITIC = new Character.UnicodeBlock("UGARITIC");
            DESERET = new Character.UnicodeBlock("DESERET");
            SHAVIAN = new Character.UnicodeBlock("SHAVIAN");
            OSMANYA = new Character.UnicodeBlock("OSMANYA");
            CYPRIOT_SYLLABARY = new Character.UnicodeBlock("CYPRIOT_SYLLABARY", new String[] { "CYPRIOT SYLLABARY", "CYPRIOTSYLLABARY" });
            BYZANTINE_MUSICAL_SYMBOLS = new Character.UnicodeBlock("BYZANTINE_MUSICAL_SYMBOLS", new String[] { "BYZANTINE MUSICAL SYMBOLS", "BYZANTINEMUSICALSYMBOLS" });
            MUSICAL_SYMBOLS = new Character.UnicodeBlock("MUSICAL_SYMBOLS", new String[] { "MUSICAL SYMBOLS", "MUSICALSYMBOLS" });
            TAI_XUAN_JING_SYMBOLS = new Character.UnicodeBlock("TAI_XUAN_JING_SYMBOLS", new String[] { "TAI XUAN JING SYMBOLS", "TAIXUANJINGSYMBOLS" });
            MATHEMATICAL_ALPHANUMERIC_SYMBOLS = new Character.UnicodeBlock("MATHEMATICAL_ALPHANUMERIC_SYMBOLS", new String[] { "MATHEMATICAL ALPHANUMERIC SYMBOLS", "MATHEMATICALALPHANUMERICSYMBOLS" });
            CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B = new Character.UnicodeBlock("CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B", new String[] { "CJK UNIFIED IDEOGRAPHS EXTENSION B", "CJKUNIFIEDIDEOGRAPHSEXTENSIONB" });
            CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT = new Character.UnicodeBlock("CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT", new String[] { "CJK COMPATIBILITY IDEOGRAPHS SUPPLEMENT", "CJKCOMPATIBILITYIDEOGRAPHSSUPPLEMENT" });
            TAGS = new Character.UnicodeBlock("TAGS");
            VARIATION_SELECTORS_SUPPLEMENT = new Character.UnicodeBlock("VARIATION_SELECTORS_SUPPLEMENT", new String[] { "VARIATION SELECTORS SUPPLEMENT", "VARIATIONSELECTORSSUPPLEMENT" });
            SUPPLEMENTARY_PRIVATE_USE_AREA_A = new Character.UnicodeBlock("SUPPLEMENTARY_PRIVATE_USE_AREA_A", new String[] { "SUPPLEMENTARY PRIVATE USE AREA-A", "SUPPLEMENTARYPRIVATEUSEAREA-A" });
            SUPPLEMENTARY_PRIVATE_USE_AREA_B = new Character.UnicodeBlock("SUPPLEMENTARY_PRIVATE_USE_AREA_B", new String[] { "SUPPLEMENTARY PRIVATE USE AREA-B", "SUPPLEMENTARYPRIVATEUSEAREA-B" });
            HIGH_SURROGATES = new Character.UnicodeBlock("HIGH_SURROGATES", new String[] { "HIGH SURROGATES", "HIGHSURROGATES" });
            HIGH_PRIVATE_USE_SURROGATES = new Character.UnicodeBlock("HIGH_PRIVATE_USE_SURROGATES", new String[] { "HIGH PRIVATE USE SURROGATES", "HIGHPRIVATEUSESURROGATES" });
            LOW_SURROGATES = new Character.UnicodeBlock("LOW_SURROGATES", new String[] { "LOW SURROGATES", "LOWSURROGATES" });
            ARABIC_SUPPLEMENT = new Character.UnicodeBlock("ARABIC_SUPPLEMENT", new String[] { "ARABIC SUPPLEMENT", "ARABICSUPPLEMENT" });
            NKO = new Character.UnicodeBlock("NKO");
            SAMARITAN = new Character.UnicodeBlock("SAMARITAN");
            MANDAIC = new Character.UnicodeBlock("MANDAIC");
            ETHIOPIC_SUPPLEMENT = new Character.UnicodeBlock("ETHIOPIC_SUPPLEMENT", new String[] { "ETHIOPIC SUPPLEMENT", "ETHIOPICSUPPLEMENT" });
            UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS_EXTENDED = new Character.UnicodeBlock("UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS_EXTENDED", new String[] { "UNIFIED CANADIAN ABORIGINAL SYLLABICS EXTENDED", "UNIFIEDCANADIANABORIGINALSYLLABICSEXTENDED" });
            NEW_TAI_LUE = new Character.UnicodeBlock("NEW_TAI_LUE", new String[] { "NEW TAI LUE", "NEWTAILUE" });
            BUGINESE = new Character.UnicodeBlock("BUGINESE");
            TAI_THAM = new Character.UnicodeBlock("TAI_THAM", new String[] { "TAI THAM", "TAITHAM" });
            BALINESE = new Character.UnicodeBlock("BALINESE");
            SUNDANESE = new Character.UnicodeBlock("SUNDANESE");
            BATAK = new Character.UnicodeBlock("BATAK");
            LEPCHA = new Character.UnicodeBlock("LEPCHA");
            OL_CHIKI = new Character.UnicodeBlock("OL_CHIKI", new String[] { "OL CHIKI", "OLCHIKI" });
            VEDIC_EXTENSIONS = new Character.UnicodeBlock("VEDIC_EXTENSIONS", new String[] { "VEDIC EXTENSIONS", "VEDICEXTENSIONS" });
            PHONETIC_EXTENSIONS_SUPPLEMENT = new Character.UnicodeBlock("PHONETIC_EXTENSIONS_SUPPLEMENT", new String[] { "PHONETIC EXTENSIONS SUPPLEMENT", "PHONETICEXTENSIONSSUPPLEMENT" });
            COMBINING_DIACRITICAL_MARKS_SUPPLEMENT = new Character.UnicodeBlock("COMBINING_DIACRITICAL_MARKS_SUPPLEMENT", new String[] { "COMBINING DIACRITICAL MARKS SUPPLEMENT", "COMBININGDIACRITICALMARKSSUPPLEMENT" });
            GLAGOLITIC = new Character.UnicodeBlock("GLAGOLITIC");
            LATIN_EXTENDED_C = new Character.UnicodeBlock("LATIN_EXTENDED_C", new String[] { "LATIN EXTENDED-C", "LATINEXTENDED-C" });
            COPTIC = new Character.UnicodeBlock("COPTIC");
            GEORGIAN_SUPPLEMENT = new Character.UnicodeBlock("GEORGIAN_SUPPLEMENT", new String[] { "GEORGIAN SUPPLEMENT", "GEORGIANSUPPLEMENT" });
            TIFINAGH = new Character.UnicodeBlock("TIFINAGH");
            ETHIOPIC_EXTENDED = new Character.UnicodeBlock("ETHIOPIC_EXTENDED", new String[] { "ETHIOPIC EXTENDED", "ETHIOPICEXTENDED" });
            CYRILLIC_EXTENDED_A = new Character.UnicodeBlock("CYRILLIC_EXTENDED_A", new String[] { "CYRILLIC EXTENDED-A", "CYRILLICEXTENDED-A" });
            SUPPLEMENTAL_PUNCTUATION = new Character.UnicodeBlock("SUPPLEMENTAL_PUNCTUATION", new String[] { "SUPPLEMENTAL PUNCTUATION", "SUPPLEMENTALPUNCTUATION" });
            CJK_STROKES = new Character.UnicodeBlock("CJK_STROKES", new String[] { "CJK STROKES", "CJKSTROKES" });
            LISU = new Character.UnicodeBlock("LISU");
            VAI = new Character.UnicodeBlock("VAI");
            CYRILLIC_EXTENDED_B = new Character.UnicodeBlock("CYRILLIC_EXTENDED_B", new String[] { "CYRILLIC EXTENDED-B", "CYRILLICEXTENDED-B" });
            BAMUM = new Character.UnicodeBlock("BAMUM");
            MODIFIER_TONE_LETTERS = new Character.UnicodeBlock("MODIFIER_TONE_LETTERS", new String[] { "MODIFIER TONE LETTERS", "MODIFIERTONELETTERS" });
            LATIN_EXTENDED_D = new Character.UnicodeBlock("LATIN_EXTENDED_D", new String[] { "LATIN EXTENDED-D", "LATINEXTENDED-D" });
            SYLOTI_NAGRI = new Character.UnicodeBlock("SYLOTI_NAGRI", new String[] { "SYLOTI NAGRI", "SYLOTINAGRI" });
            COMMON_INDIC_NUMBER_FORMS = new Character.UnicodeBlock("COMMON_INDIC_NUMBER_FORMS", new String[] { "COMMON INDIC NUMBER FORMS", "COMMONINDICNUMBERFORMS" });
            PHAGS_PA = new Character.UnicodeBlock("PHAGS_PA", "PHAGS-PA");
            SAURASHTRA = new Character.UnicodeBlock("SAURASHTRA");
            DEVANAGARI_EXTENDED = new Character.UnicodeBlock("DEVANAGARI_EXTENDED", new String[] { "DEVANAGARI EXTENDED", "DEVANAGARIEXTENDED" });
            KAYAH_LI = new Character.UnicodeBlock("KAYAH_LI", new String[] { "KAYAH LI", "KAYAHLI" });
            REJANG = new Character.UnicodeBlock("REJANG");
            HANGUL_JAMO_EXTENDED_A = new Character.UnicodeBlock("HANGUL_JAMO_EXTENDED_A", new String[] { "HANGUL JAMO EXTENDED-A", "HANGULJAMOEXTENDED-A" });
            JAVANESE = new Character.UnicodeBlock("JAVANESE");
            CHAM = new Character.UnicodeBlock("CHAM");
            MYANMAR_EXTENDED_A = new Character.UnicodeBlock("MYANMAR_EXTENDED_A", new String[] { "MYANMAR EXTENDED-A", "MYANMAREXTENDED-A" });
            TAI_VIET = new Character.UnicodeBlock("TAI_VIET", new String[] { "TAI VIET", "TAIVIET" });
            ETHIOPIC_EXTENDED_A = new Character.UnicodeBlock("ETHIOPIC_EXTENDED_A", new String[] { "ETHIOPIC EXTENDED-A", "ETHIOPICEXTENDED-A" });
            MEETEI_MAYEK = new Character.UnicodeBlock("MEETEI_MAYEK", new String[] { "MEETEI MAYEK", "MEETEIMAYEK" });
            HANGUL_JAMO_EXTENDED_B = new Character.UnicodeBlock("HANGUL_JAMO_EXTENDED_B", new String[] { "HANGUL JAMO EXTENDED-B", "HANGULJAMOEXTENDED-B" });
            VERTICAL_FORMS = new Character.UnicodeBlock("VERTICAL_FORMS", new String[] { "VERTICAL FORMS", "VERTICALFORMS" });
            ANCIENT_GREEK_NUMBERS = new Character.UnicodeBlock("ANCIENT_GREEK_NUMBERS", new String[] { "ANCIENT GREEK NUMBERS", "ANCIENTGREEKNUMBERS" });
            ANCIENT_SYMBOLS = new Character.UnicodeBlock("ANCIENT_SYMBOLS", new String[] { "ANCIENT SYMBOLS", "ANCIENTSYMBOLS" });
            PHAISTOS_DISC = new Character.UnicodeBlock("PHAISTOS_DISC", new String[] { "PHAISTOS DISC", "PHAISTOSDISC" });
            LYCIAN = new Character.UnicodeBlock("LYCIAN");
            CARIAN = new Character.UnicodeBlock("CARIAN");
            OLD_PERSIAN = new Character.UnicodeBlock("OLD_PERSIAN", new String[] { "OLD PERSIAN", "OLDPERSIAN" });
            IMPERIAL_ARAMAIC = new Character.UnicodeBlock("IMPERIAL_ARAMAIC", new String[] { "IMPERIAL ARAMAIC", "IMPERIALARAMAIC" });
            PHOENICIAN = new Character.UnicodeBlock("PHOENICIAN");
            LYDIAN = new Character.UnicodeBlock("LYDIAN");
            KHAROSHTHI = new Character.UnicodeBlock("KHAROSHTHI");
            OLD_SOUTH_ARABIAN = new Character.UnicodeBlock("OLD_SOUTH_ARABIAN", new String[] { "OLD SOUTH ARABIAN", "OLDSOUTHARABIAN" });
            AVESTAN = new Character.UnicodeBlock("AVESTAN");
            INSCRIPTIONAL_PARTHIAN = new Character.UnicodeBlock("INSCRIPTIONAL_PARTHIAN", new String[] { "INSCRIPTIONAL PARTHIAN", "INSCRIPTIONALPARTHIAN" });
            INSCRIPTIONAL_PAHLAVI = new Character.UnicodeBlock("INSCRIPTIONAL_PAHLAVI", new String[] { "INSCRIPTIONAL PAHLAVI", "INSCRIPTIONALPAHLAVI" });
            OLD_TURKIC = new Character.UnicodeBlock("OLD_TURKIC", new String[] { "OLD TURKIC", "OLDTURKIC" });
            RUMI_NUMERAL_SYMBOLS = new Character.UnicodeBlock("RUMI_NUMERAL_SYMBOLS", new String[] { "RUMI NUMERAL SYMBOLS", "RUMINUMERALSYMBOLS" });
            BRAHMI = new Character.UnicodeBlock("BRAHMI");
            KAITHI = new Character.UnicodeBlock("KAITHI");
            CUNEIFORM = new Character.UnicodeBlock("CUNEIFORM");
            CUNEIFORM_NUMBERS_AND_PUNCTUATION = new Character.UnicodeBlock("CUNEIFORM_NUMBERS_AND_PUNCTUATION", new String[] { "CUNEIFORM NUMBERS AND PUNCTUATION", "CUNEIFORMNUMBERSANDPUNCTUATION" });
            EGYPTIAN_HIEROGLYPHS = new Character.UnicodeBlock("EGYPTIAN_HIEROGLYPHS", new String[] { "EGYPTIAN HIEROGLYPHS", "EGYPTIANHIEROGLYPHS" });
            BAMUM_SUPPLEMENT = new Character.UnicodeBlock("BAMUM_SUPPLEMENT", new String[] { "BAMUM SUPPLEMENT", "BAMUMSUPPLEMENT" });
            KANA_SUPPLEMENT = new Character.UnicodeBlock("KANA_SUPPLEMENT", new String[] { "KANA SUPPLEMENT", "KANASUPPLEMENT" });
            ANCIENT_GREEK_MUSICAL_NOTATION = new Character.UnicodeBlock("ANCIENT_GREEK_MUSICAL_NOTATION", new String[] { "ANCIENT GREEK MUSICAL NOTATION", "ANCIENTGREEKMUSICALNOTATION" });
            COUNTING_ROD_NUMERALS = new Character.UnicodeBlock("COUNTING_ROD_NUMERALS", new String[] { "COUNTING ROD NUMERALS", "COUNTINGRODNUMERALS" });
            MAHJONG_TILES = new Character.UnicodeBlock("MAHJONG_TILES", new String[] { "MAHJONG TILES", "MAHJONGTILES" });
            DOMINO_TILES = new Character.UnicodeBlock("DOMINO_TILES", new String[] { "DOMINO TILES", "DOMINOTILES" });
            PLAYING_CARDS = new Character.UnicodeBlock("PLAYING_CARDS", new String[] { "PLAYING CARDS", "PLAYINGCARDS" });
            ENCLOSED_ALPHANUMERIC_SUPPLEMENT = new Character.UnicodeBlock("ENCLOSED_ALPHANUMERIC_SUPPLEMENT", new String[] { "ENCLOSED ALPHANUMERIC SUPPLEMENT", "ENCLOSEDALPHANUMERICSUPPLEMENT" });
            ENCLOSED_IDEOGRAPHIC_SUPPLEMENT = new Character.UnicodeBlock("ENCLOSED_IDEOGRAPHIC_SUPPLEMENT", new String[] { "ENCLOSED IDEOGRAPHIC SUPPLEMENT", "ENCLOSEDIDEOGRAPHICSUPPLEMENT" });
            MISCELLANEOUS_SYMBOLS_AND_PICTOGRAPHS = new Character.UnicodeBlock("MISCELLANEOUS_SYMBOLS_AND_PICTOGRAPHS", new String[] { "MISCELLANEOUS SYMBOLS AND PICTOGRAPHS", "MISCELLANEOUSSYMBOLSANDPICTOGRAPHS" });
            EMOTICONS = new Character.UnicodeBlock("EMOTICONS");
            TRANSPORT_AND_MAP_SYMBOLS = new Character.UnicodeBlock("TRANSPORT_AND_MAP_SYMBOLS", new String[] { "TRANSPORT AND MAP SYMBOLS", "TRANSPORTANDMAPSYMBOLS" });
            ALCHEMICAL_SYMBOLS = new Character.UnicodeBlock("ALCHEMICAL_SYMBOLS", new String[] { "ALCHEMICAL SYMBOLS", "ALCHEMICALSYMBOLS" });
            CJK_UNIFIED_IDEOGRAPHS_EXTENSION_C = new Character.UnicodeBlock("CJK_UNIFIED_IDEOGRAPHS_EXTENSION_C", new String[] { "CJK UNIFIED IDEOGRAPHS EXTENSION C", "CJKUNIFIEDIDEOGRAPHSEXTENSIONC" });
            CJK_UNIFIED_IDEOGRAPHS_EXTENSION_D = new Character.UnicodeBlock("CJK_UNIFIED_IDEOGRAPHS_EXTENSION_D", new String[] { "CJK UNIFIED IDEOGRAPHS EXTENSION D", "CJKUNIFIEDIDEOGRAPHSEXTENSIOND" });
            ARABIC_EXTENDED_A = new Character.UnicodeBlock("ARABIC_EXTENDED_A", new String[] { "ARABIC EXTENDED-A", "ARABICEXTENDED-A" });
            SUNDANESE_SUPPLEMENT = new Character.UnicodeBlock("SUNDANESE_SUPPLEMENT", new String[] { "SUNDANESE SUPPLEMENT", "SUNDANESESUPPLEMENT" });
            MEETEI_MAYEK_EXTENSIONS = new Character.UnicodeBlock("MEETEI_MAYEK_EXTENSIONS", new String[] { "MEETEI MAYEK EXTENSIONS", "MEETEIMAYEKEXTENSIONS" });
            MEROITIC_HIEROGLYPHS = new Character.UnicodeBlock("MEROITIC_HIEROGLYPHS", new String[] { "MEROITIC HIEROGLYPHS", "MEROITICHIEROGLYPHS" });
            MEROITIC_CURSIVE = new Character.UnicodeBlock("MEROITIC_CURSIVE", new String[] { "MEROITIC CURSIVE", "MEROITICCURSIVE" });
            SORA_SOMPENG = new Character.UnicodeBlock("SORA_SOMPENG", new String[] { "SORA SOMPENG", "SORASOMPENG" });
            CHAKMA = new Character.UnicodeBlock("CHAKMA");
            SHARADA = new Character.UnicodeBlock("SHARADA");
            TAKRI = new Character.UnicodeBlock("TAKRI");
            MIAO = new Character.UnicodeBlock("MIAO");
            ARABIC_MATHEMATICAL_ALPHABETIC_SYMBOLS = new Character.UnicodeBlock("ARABIC_MATHEMATICAL_ALPHABETIC_SYMBOLS", new String[] { "ARABIC MATHEMATICAL ALPHABETIC SYMBOLS", "ARABICMATHEMATICALALPHABETICSYMBOLS" });
            blockStarts = new int[] { 0, 128, 256, 384, 592, 688, 768, 880, 1024, 1280, 1328, 1424, 1536, 1792, 1872, 1920, 1984, 2048, 2112, 2144, 2208, 2304, 2432, 2560, 2688, 2816, 2944, 3072, 3200, 3328, 3456, 3584, 3712, 3840, 4096, 4256, 4352, 4608, 4992, 5024, 5120, 5760, 5792, 5888, 5920, 5952, 5984, 6016, 6144, 6320, 6400, 6480, 6528, 6624, 6656, 6688, 6832, 6912, 7040, 7104, 7168, 7248, 7296, 7360, 7376, 7424, 7552, 7616, 7680, 7936, 8192, 8304, 8352, 8400, 8448, 8528, 8592, 8704, 8960, 9216, 9280, 9312, 9472, 9600, 9632, 9728, 9984, 10176, 10224, 10240, 10496, 10624, 10752, 11008, 11264, 11360, 11392, 11520, 11568, 11648, 11744, 11776, 11904, 12032, 12256, 12272, 12288, 12352, 12448, 12544, 12592, 12688, 12704, 12736, 12784, 12800, 13056, 13312, 19904, 19968, 40960, 42128, 42192, 42240, 42560, 42656, 42752, 42784, 43008, 43056, 43072, 43136, 43232, 43264, 43312, 43360, 43392, 43488, 43520, 43616, 43648, 43744, 43776, 43824, 43968, 44032, 55216, 55296, 56192, 56320, 57344, 63744, 64256, 64336, 65024, 65040, 65056, 65072, 65104, 65136, 65280, 65520, 65536, 65664, 65792, 65856, 65936, 66000, 66048, 66176, 66208, 66272, 66304, 66352, 66384, 66432, 66464, 66528, 66560, 66640, 66688, 66736, 67584, 67648, 67680, 67840, 67872, 67904, 67968, 68000, 68096, 68192, 68224, 68352, 68416, 68448, 68480, 68608, 68688, 69216, 69248, 69632, 69760, 69840, 69888, 69968, 70016, 70112, 71296, 71376, 73728, 74752, 74880, 77824, 78896, 92160, 92736, 93952, 94112, 110592, 110848, 118784, 119040, 119296, 119376, 119552, 119648, 119680, 119808, 120832, 126464, 126720, 126976, 127024, 127136, 127232, 127488, 127744, 128512, 128592, 128640, 128768, 128896, 131072, 173792, 173824, 177984, 178208, 194560, 195104, 917504, 917632, 917760, 918000, 983040, 1048576 };
            blocks = new Character.UnicodeBlock[] { Character.UnicodeBlock.BASIC_LATIN, Character.UnicodeBlock.LATIN_1_SUPPLEMENT, Character.UnicodeBlock.LATIN_EXTENDED_A, Character.UnicodeBlock.LATIN_EXTENDED_B, Character.UnicodeBlock.IPA_EXTENSIONS, Character.UnicodeBlock.SPACING_MODIFIER_LETTERS, Character.UnicodeBlock.COMBINING_DIACRITICAL_MARKS, Character.UnicodeBlock.GREEK, Character.UnicodeBlock.CYRILLIC, Character.UnicodeBlock.CYRILLIC_SUPPLEMENTARY, Character.UnicodeBlock.ARMENIAN, Character.UnicodeBlock.HEBREW, Character.UnicodeBlock.ARABIC, Character.UnicodeBlock.SYRIAC, Character.UnicodeBlock.ARABIC_SUPPLEMENT, Character.UnicodeBlock.THAANA, Character.UnicodeBlock.NKO, Character.UnicodeBlock.SAMARITAN, Character.UnicodeBlock.MANDAIC, null, Character.UnicodeBlock.ARABIC_EXTENDED_A, Character.UnicodeBlock.DEVANAGARI, Character.UnicodeBlock.BENGALI, Character.UnicodeBlock.GURMUKHI, Character.UnicodeBlock.GUJARATI, Character.UnicodeBlock.ORIYA, Character.UnicodeBlock.TAMIL, Character.UnicodeBlock.TELUGU, Character.UnicodeBlock.KANNADA, Character.UnicodeBlock.MALAYALAM, Character.UnicodeBlock.SINHALA, Character.UnicodeBlock.THAI, Character.UnicodeBlock.LAO, Character.UnicodeBlock.TIBETAN, Character.UnicodeBlock.MYANMAR, Character.UnicodeBlock.GEORGIAN, Character.UnicodeBlock.HANGUL_JAMO, Character.UnicodeBlock.ETHIOPIC, Character.UnicodeBlock.ETHIOPIC_SUPPLEMENT, Character.UnicodeBlock.CHEROKEE, Character.UnicodeBlock.UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS, Character.UnicodeBlock.OGHAM, Character.UnicodeBlock.RUNIC, Character.UnicodeBlock.TAGALOG, Character.UnicodeBlock.HANUNOO, Character.UnicodeBlock.BUHID, Character.UnicodeBlock.TAGBANWA, Character.UnicodeBlock.KHMER, Character.UnicodeBlock.MONGOLIAN, Character.UnicodeBlock.UNIFIED_CANADIAN_ABORIGINAL_SYLLABICS_EXTENDED, Character.UnicodeBlock.LIMBU, Character.UnicodeBlock.TAI_LE, Character.UnicodeBlock.NEW_TAI_LUE, Character.UnicodeBlock.KHMER_SYMBOLS, Character.UnicodeBlock.BUGINESE, Character.UnicodeBlock.TAI_THAM, null, Character.UnicodeBlock.BALINESE, Character.UnicodeBlock.SUNDANESE, Character.UnicodeBlock.BATAK, Character.UnicodeBlock.LEPCHA, Character.UnicodeBlock.OL_CHIKI, null, Character.UnicodeBlock.SUNDANESE_SUPPLEMENT, Character.UnicodeBlock.VEDIC_EXTENSIONS, Character.UnicodeBlock.PHONETIC_EXTENSIONS, Character.UnicodeBlock.PHONETIC_EXTENSIONS_SUPPLEMENT, Character.UnicodeBlock.COMBINING_DIACRITICAL_MARKS_SUPPLEMENT, Character.UnicodeBlock.LATIN_EXTENDED_ADDITIONAL, Character.UnicodeBlock.GREEK_EXTENDED, Character.UnicodeBlock.GENERAL_PUNCTUATION, Character.UnicodeBlock.SUPERSCRIPTS_AND_SUBSCRIPTS, Character.UnicodeBlock.CURRENCY_SYMBOLS, Character.UnicodeBlock.COMBINING_MARKS_FOR_SYMBOLS, Character.UnicodeBlock.LETTERLIKE_SYMBOLS, Character.UnicodeBlock.NUMBER_FORMS, Character.UnicodeBlock.ARROWS, Character.UnicodeBlock.MATHEMATICAL_OPERATORS, Character.UnicodeBlock.MISCELLANEOUS_TECHNICAL, Character.UnicodeBlock.CONTROL_PICTURES, Character.UnicodeBlock.OPTICAL_CHARACTER_RECOGNITION, Character.UnicodeBlock.ENCLOSED_ALPHANUMERICS, Character.UnicodeBlock.BOX_DRAWING, Character.UnicodeBlock.BLOCK_ELEMENTS, Character.UnicodeBlock.GEOMETRIC_SHAPES, Character.UnicodeBlock.MISCELLANEOUS_SYMBOLS, Character.UnicodeBlock.DINGBATS, Character.UnicodeBlock.MISCELLANEOUS_MATHEMATICAL_SYMBOLS_A, Character.UnicodeBlock.SUPPLEMENTAL_ARROWS_A, Character.UnicodeBlock.BRAILLE_PATTERNS, Character.UnicodeBlock.SUPPLEMENTAL_ARROWS_B, Character.UnicodeBlock.MISCELLANEOUS_MATHEMATICAL_SYMBOLS_B, Character.UnicodeBlock.SUPPLEMENTAL_MATHEMATICAL_OPERATORS, Character.UnicodeBlock.MISCELLANEOUS_SYMBOLS_AND_ARROWS, Character.UnicodeBlock.GLAGOLITIC, Character.UnicodeBlock.LATIN_EXTENDED_C, Character.UnicodeBlock.COPTIC, Character.UnicodeBlock.GEORGIAN_SUPPLEMENT, Character.UnicodeBlock.TIFINAGH, Character.UnicodeBlock.ETHIOPIC_EXTENDED, Character.UnicodeBlock.CYRILLIC_EXTENDED_A, Character.UnicodeBlock.SUPPLEMENTAL_PUNCTUATION, Character.UnicodeBlock.CJK_RADICALS_SUPPLEMENT, Character.UnicodeBlock.KANGXI_RADICALS, null, Character.UnicodeBlock.IDEOGRAPHIC_DESCRIPTION_CHARACTERS, Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION, Character.UnicodeBlock.HIRAGANA, Character.UnicodeBlock.KATAKANA, Character.UnicodeBlock.BOPOMOFO, Character.UnicodeBlock.HANGUL_COMPATIBILITY_JAMO, Character.UnicodeBlock.KANBUN, Character.UnicodeBlock.BOPOMOFO_EXTENDED, Character.UnicodeBlock.CJK_STROKES, Character.UnicodeBlock.KATAKANA_PHONETIC_EXTENSIONS, Character.UnicodeBlock.ENCLOSED_CJK_LETTERS_AND_MONTHS, Character.UnicodeBlock.CJK_COMPATIBILITY, Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A, Character.UnicodeBlock.YIJING_HEXAGRAM_SYMBOLS, Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS, Character.UnicodeBlock.YI_SYLLABLES, Character.UnicodeBlock.YI_RADICALS, Character.UnicodeBlock.LISU, Character.UnicodeBlock.VAI, Character.UnicodeBlock.CYRILLIC_EXTENDED_B, Character.UnicodeBlock.BAMUM, Character.UnicodeBlock.MODIFIER_TONE_LETTERS, Character.UnicodeBlock.LATIN_EXTENDED_D, Character.UnicodeBlock.SYLOTI_NAGRI, Character.UnicodeBlock.COMMON_INDIC_NUMBER_FORMS, Character.UnicodeBlock.PHAGS_PA, Character.UnicodeBlock.SAURASHTRA, Character.UnicodeBlock.DEVANAGARI_EXTENDED, Character.UnicodeBlock.KAYAH_LI, Character.UnicodeBlock.REJANG, Character.UnicodeBlock.HANGUL_JAMO_EXTENDED_A, Character.UnicodeBlock.JAVANESE, null, Character.UnicodeBlock.CHAM, Character.UnicodeBlock.MYANMAR_EXTENDED_A, Character.UnicodeBlock.TAI_VIET, Character.UnicodeBlock.MEETEI_MAYEK_EXTENSIONS, Character.UnicodeBlock.ETHIOPIC_EXTENDED_A, null, Character.UnicodeBlock.MEETEI_MAYEK, Character.UnicodeBlock.HANGUL_SYLLABLES, Character.UnicodeBlock.HANGUL_JAMO_EXTENDED_B, Character.UnicodeBlock.HIGH_SURROGATES, Character.UnicodeBlock.HIGH_PRIVATE_USE_SURROGATES, Character.UnicodeBlock.LOW_SURROGATES, Character.UnicodeBlock.PRIVATE_USE_AREA, Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS, Character.UnicodeBlock.ALPHABETIC_PRESENTATION_FORMS, Character.UnicodeBlock.ARABIC_PRESENTATION_FORMS_A, Character.UnicodeBlock.VARIATION_SELECTORS, Character.UnicodeBlock.VERTICAL_FORMS, Character.UnicodeBlock.COMBINING_HALF_MARKS, Character.UnicodeBlock.CJK_COMPATIBILITY_FORMS, Character.UnicodeBlock.SMALL_FORM_VARIANTS, Character.UnicodeBlock.ARABIC_PRESENTATION_FORMS_B, Character.UnicodeBlock.HALFWIDTH_AND_FULLWIDTH_FORMS, Character.UnicodeBlock.SPECIALS, Character.UnicodeBlock.LINEAR_B_SYLLABARY, Character.UnicodeBlock.LINEAR_B_IDEOGRAMS, Character.UnicodeBlock.AEGEAN_NUMBERS, Character.UnicodeBlock.ANCIENT_GREEK_NUMBERS, Character.UnicodeBlock.ANCIENT_SYMBOLS, Character.UnicodeBlock.PHAISTOS_DISC, null, Character.UnicodeBlock.LYCIAN, Character.UnicodeBlock.CARIAN, null, Character.UnicodeBlock.OLD_ITALIC, Character.UnicodeBlock.GOTHIC, null, Character.UnicodeBlock.UGARITIC, Character.UnicodeBlock.OLD_PERSIAN, null, Character.UnicodeBlock.DESERET, Character.UnicodeBlock.SHAVIAN, Character.UnicodeBlock.OSMANYA, null, Character.UnicodeBlock.CYPRIOT_SYLLABARY, Character.UnicodeBlock.IMPERIAL_ARAMAIC, null, Character.UnicodeBlock.PHOENICIAN, Character.UnicodeBlock.LYDIAN, null, Character.UnicodeBlock.MEROITIC_HIEROGLYPHS, Character.UnicodeBlock.MEROITIC_CURSIVE, Character.UnicodeBlock.KHAROSHTHI, Character.UnicodeBlock.OLD_SOUTH_ARABIAN, null, Character.UnicodeBlock.AVESTAN, Character.UnicodeBlock.INSCRIPTIONAL_PARTHIAN, Character.UnicodeBlock.INSCRIPTIONAL_PAHLAVI, null, Character.UnicodeBlock.OLD_TURKIC, null, Character.UnicodeBlock.RUMI_NUMERAL_SYMBOLS, null, Character.UnicodeBlock.BRAHMI, Character.UnicodeBlock.KAITHI, Character.UnicodeBlock.SORA_SOMPENG, Character.UnicodeBlock.CHAKMA, null, Character.UnicodeBlock.SHARADA, null, Character.UnicodeBlock.TAKRI, null, Character.UnicodeBlock.CUNEIFORM, Character.UnicodeBlock.CUNEIFORM_NUMBERS_AND_PUNCTUATION, null, Character.UnicodeBlock.EGYPTIAN_HIEROGLYPHS, null, Character.UnicodeBlock.BAMUM_SUPPLEMENT, null, Character.UnicodeBlock.MIAO, null, Character.UnicodeBlock.KANA_SUPPLEMENT, null, Character.UnicodeBlock.BYZANTINE_MUSICAL_SYMBOLS, Character.UnicodeBlock.MUSICAL_SYMBOLS, Character.UnicodeBlock.ANCIENT_GREEK_MUSICAL_NOTATION, null, Character.UnicodeBlock.TAI_XUAN_JING_SYMBOLS, Character.UnicodeBlock.COUNTING_ROD_NUMERALS, null, Character.UnicodeBlock.MATHEMATICAL_ALPHANUMERIC_SYMBOLS, null, Character.UnicodeBlock.ARABIC_MATHEMATICAL_ALPHABETIC_SYMBOLS, null, Character.UnicodeBlock.MAHJONG_TILES, Character.UnicodeBlock.DOMINO_TILES, Character.UnicodeBlock.PLAYING_CARDS, Character.UnicodeBlock.ENCLOSED_ALPHANUMERIC_SUPPLEMENT, Character.UnicodeBlock.ENCLOSED_IDEOGRAPHIC_SUPPLEMENT, Character.UnicodeBlock.MISCELLANEOUS_SYMBOLS_AND_PICTOGRAPHS, Character.UnicodeBlock.EMOTICONS, null, Character.UnicodeBlock.TRANSPORT_AND_MAP_SYMBOLS, Character.UnicodeBlock.ALCHEMICAL_SYMBOLS, null, Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B, null, Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_C, Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_D, null, Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS_SUPPLEMENT, null, Character.UnicodeBlock.TAGS, null, Character.UnicodeBlock.VARIATION_SELECTORS_SUPPLEMENT, null, Character.UnicodeBlock.SUPPLEMENTARY_PRIVATE_USE_AREA_A, Character.UnicodeBlock.SUPPLEMENTARY_PRIVATE_USE_AREA_B };
        }
    }
    
    public enum UnicodeScript
    {
        COMMON, 
        LATIN, 
        GREEK, 
        CYRILLIC, 
        ARMENIAN, 
        HEBREW, 
        ARABIC, 
        SYRIAC, 
        THAANA, 
        DEVANAGARI, 
        BENGALI, 
        GURMUKHI, 
        GUJARATI, 
        ORIYA, 
        TAMIL, 
        TELUGU, 
        KANNADA, 
        MALAYALAM, 
        SINHALA, 
        THAI, 
        LAO, 
        TIBETAN, 
        MYANMAR, 
        GEORGIAN, 
        HANGUL, 
        ETHIOPIC, 
        CHEROKEE, 
        CANADIAN_ABORIGINAL, 
        OGHAM, 
        RUNIC, 
        KHMER, 
        MONGOLIAN, 
        HIRAGANA, 
        KATAKANA, 
        BOPOMOFO, 
        HAN, 
        YI, 
        OLD_ITALIC, 
        GOTHIC, 
        DESERET, 
        INHERITED, 
        TAGALOG, 
        HANUNOO, 
        BUHID, 
        TAGBANWA, 
        LIMBU, 
        TAI_LE, 
        LINEAR_B, 
        UGARITIC, 
        SHAVIAN, 
        OSMANYA, 
        CYPRIOT, 
        BRAILLE, 
        BUGINESE, 
        COPTIC, 
        NEW_TAI_LUE, 
        GLAGOLITIC, 
        TIFINAGH, 
        SYLOTI_NAGRI, 
        OLD_PERSIAN, 
        KHAROSHTHI, 
        BALINESE, 
        CUNEIFORM, 
        PHOENICIAN, 
        PHAGS_PA, 
        NKO, 
        SUNDANESE, 
        BATAK, 
        LEPCHA, 
        OL_CHIKI, 
        VAI, 
        SAURASHTRA, 
        KAYAH_LI, 
        REJANG, 
        LYCIAN, 
        CARIAN, 
        LYDIAN, 
        CHAM, 
        TAI_THAM, 
        TAI_VIET, 
        AVESTAN, 
        EGYPTIAN_HIEROGLYPHS, 
        SAMARITAN, 
        MANDAIC, 
        LISU, 
        BAMUM, 
        JAVANESE, 
        MEETEI_MAYEK, 
        IMPERIAL_ARAMAIC, 
        OLD_SOUTH_ARABIAN, 
        INSCRIPTIONAL_PARTHIAN, 
        INSCRIPTIONAL_PAHLAVI, 
        OLD_TURKIC, 
        BRAHMI, 
        KAITHI, 
        MEROITIC_HIEROGLYPHS, 
        MEROITIC_CURSIVE, 
        SORA_SOMPENG, 
        CHAKMA, 
        SHARADA, 
        TAKRI, 
        MIAO, 
        UNKNOWN;
        
        private static final int[] scriptStarts;
        private static final Character.UnicodeScript[] scripts;
        private static HashMap<String, Character.UnicodeScript> aliases;
        
        public static Character.UnicodeScript of(int key) {
            if (!Character.isValidCodePoint(key)) {
                throw new IllegalArgumentException();
            }
            if (Character.getType(key) == 0) {
                return Character.UnicodeScript.UNKNOWN;
            }
            int binarySearch = Arrays.binarySearch(Character.UnicodeScript.scriptStarts, key);
            if (binarySearch < 0) {
                binarySearch = -binarySearch - 2;
            }
            return Character.UnicodeScript.scripts[binarySearch];
        }
        
        public static final Character.UnicodeScript forName(String upperCase) {
            upperCase = upperCase.toUpperCase(Locale.ENGLISH);
            Character.UnicodeScript unicodeScript = (Character.UnicodeScript)Character.UnicodeScript.aliases.get(upperCase);
            if (unicodeScript != null) {
                return unicodeScript;
            }
            return valueOf(upperCase);
        }
        
        static {
            scriptStarts = new int[] { 0, 65, 91, 97, 123, 170, 171, 186, 187, 192, 215, 216, 247, 248, 697, 736, 741, 746, 748, 768, 880, 884, 885, 894, 900, 901, 902, 903, 904, 994, 1008, 1024, 1157, 1159, 1329, 1417, 1418, 1425, 1536, 1548, 1549, 1563, 1566, 1567, 1568, 1600, 1601, 1611, 1622, 1632, 1642, 1648, 1649, 1757, 1758, 1792, 1872, 1920, 1984, 2048, 2112, 2208, 2304, 2385, 2387, 2404, 2406, 2433, 2561, 2689, 2817, 2946, 3073, 3202, 3330, 3458, 3585, 3647, 3648, 3713, 3840, 4053, 4057, 4096, 4256, 4347, 4348, 4352, 4608, 5024, 5120, 5760, 5792, 5867, 5870, 5888, 5920, 5941, 5952, 5984, 6016, 6144, 6146, 6148, 6149, 6150, 6320, 6400, 6480, 6528, 6624, 6656, 6688, 6912, 7040, 7104, 7168, 7248, 7360, 7376, 7379, 7380, 7393, 7394, 7401, 7405, 7406, 7412, 7413, 7424, 7462, 7467, 7468, 7517, 7522, 7526, 7531, 7544, 7545, 7615, 7616, 7680, 7936, 8192, 8204, 8206, 8305, 8308, 8319, 8320, 8336, 8352, 8400, 8448, 8486, 8487, 8490, 8492, 8498, 8499, 8526, 8527, 8544, 8585, 10240, 10496, 11264, 11360, 11392, 11520, 11568, 11648, 11744, 11776, 11904, 12272, 12293, 12294, 12295, 12296, 12321, 12330, 12334, 12336, 12344, 12348, 12353, 12441, 12443, 12445, 12448, 12449, 12539, 12541, 12549, 12593, 12688, 12704, 12736, 12784, 12800, 12832, 12896, 12927, 13008, 13055, 13056, 13144, 13312, 19904, 19968, 40960, 42192, 42240, 42560, 42656, 42752, 42786, 42888, 42891, 43008, 43056, 43072, 43136, 43232, 43264, 43312, 43360, 43392, 43520, 43616, 43648, 43744, 43777, 43968, 44032, 55292, 63744, 64256, 64275, 64285, 64336, 64830, 64848, 65021, 65024, 65040, 65056, 65072, 65136, 65279, 65313, 65339, 65345, 65371, 65382, 65392, 65393, 65438, 65440, 65504, 65536, 65792, 65856, 65936, 66045, 66176, 66208, 66304, 66352, 66432, 66464, 66560, 66640, 66688, 67584, 67648, 67840, 67872, 67968, 68000, 68096, 68192, 68352, 68416, 68448, 68608, 69216, 69632, 69760, 69840, 69888, 70016, 71296, 73728, 77824, 92160, 93952, 110592, 110593, 118784, 119143, 119146, 119163, 119171, 119173, 119180, 119210, 119214, 119296, 119552, 126464, 126976, 127488, 127489, 131072, 917505, 917760, 918000 };
            scripts = new Character.UnicodeScript[] { Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.BOPOMOFO, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.GREEK, Character.UnicodeScript.COPTIC, Character.UnicodeScript.GREEK, Character.UnicodeScript.CYRILLIC, Character.UnicodeScript.INHERITED, Character.UnicodeScript.CYRILLIC, Character.UnicodeScript.ARMENIAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARMENIAN, Character.UnicodeScript.HEBREW, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.INHERITED, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.INHERITED, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.SYRIAC, Character.UnicodeScript.ARABIC, Character.UnicodeScript.THAANA, Character.UnicodeScript.NKO, Character.UnicodeScript.SAMARITAN, Character.UnicodeScript.MANDAIC, Character.UnicodeScript.ARABIC, Character.UnicodeScript.DEVANAGARI, Character.UnicodeScript.INHERITED, Character.UnicodeScript.DEVANAGARI, Character.UnicodeScript.COMMON, Character.UnicodeScript.DEVANAGARI, Character.UnicodeScript.BENGALI, Character.UnicodeScript.GURMUKHI, Character.UnicodeScript.GUJARATI, Character.UnicodeScript.ORIYA, Character.UnicodeScript.TAMIL, Character.UnicodeScript.TELUGU, Character.UnicodeScript.KANNADA, Character.UnicodeScript.MALAYALAM, Character.UnicodeScript.SINHALA, Character.UnicodeScript.THAI, Character.UnicodeScript.COMMON, Character.UnicodeScript.THAI, Character.UnicodeScript.LAO, Character.UnicodeScript.TIBETAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.TIBETAN, Character.UnicodeScript.MYANMAR, Character.UnicodeScript.GEORGIAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.GEORGIAN, Character.UnicodeScript.HANGUL, Character.UnicodeScript.ETHIOPIC, Character.UnicodeScript.CHEROKEE, Character.UnicodeScript.CANADIAN_ABORIGINAL, Character.UnicodeScript.OGHAM, Character.UnicodeScript.RUNIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.RUNIC, Character.UnicodeScript.TAGALOG, Character.UnicodeScript.HANUNOO, Character.UnicodeScript.COMMON, Character.UnicodeScript.BUHID, Character.UnicodeScript.TAGBANWA, Character.UnicodeScript.KHMER, Character.UnicodeScript.MONGOLIAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.MONGOLIAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.MONGOLIAN, Character.UnicodeScript.CANADIAN_ABORIGINAL, Character.UnicodeScript.LIMBU, Character.UnicodeScript.TAI_LE, Character.UnicodeScript.NEW_TAI_LUE, Character.UnicodeScript.KHMER, Character.UnicodeScript.BUGINESE, Character.UnicodeScript.TAI_THAM, Character.UnicodeScript.BALINESE, Character.UnicodeScript.SUNDANESE, Character.UnicodeScript.BATAK, Character.UnicodeScript.LEPCHA, Character.UnicodeScript.OL_CHIKI, Character.UnicodeScript.SUNDANESE, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.GREEK, Character.UnicodeScript.CYRILLIC, Character.UnicodeScript.LATIN, Character.UnicodeScript.GREEK, Character.UnicodeScript.LATIN, Character.UnicodeScript.GREEK, Character.UnicodeScript.LATIN, Character.UnicodeScript.CYRILLIC, Character.UnicodeScript.LATIN, Character.UnicodeScript.GREEK, Character.UnicodeScript.INHERITED, Character.UnicodeScript.LATIN, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.BRAILLE, Character.UnicodeScript.COMMON, Character.UnicodeScript.GLAGOLITIC, Character.UnicodeScript.LATIN, Character.UnicodeScript.COPTIC, Character.UnicodeScript.GEORGIAN, Character.UnicodeScript.TIFINAGH, Character.UnicodeScript.ETHIOPIC, Character.UnicodeScript.CYRILLIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.INHERITED, Character.UnicodeScript.HANGUL, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.HIRAGANA, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.HIRAGANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.BOPOMOFO, Character.UnicodeScript.HANGUL, Character.UnicodeScript.COMMON, Character.UnicodeScript.BOPOMOFO, Character.UnicodeScript.COMMON, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.HANGUL, Character.UnicodeScript.COMMON, Character.UnicodeScript.HANGUL, Character.UnicodeScript.COMMON, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.YI, Character.UnicodeScript.LISU, Character.UnicodeScript.VAI, Character.UnicodeScript.CYRILLIC, Character.UnicodeScript.BAMUM, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.SYLOTI_NAGRI, Character.UnicodeScript.COMMON, Character.UnicodeScript.PHAGS_PA, Character.UnicodeScript.SAURASHTRA, Character.UnicodeScript.DEVANAGARI, Character.UnicodeScript.KAYAH_LI, Character.UnicodeScript.REJANG, Character.UnicodeScript.HANGUL, Character.UnicodeScript.JAVANESE, Character.UnicodeScript.CHAM, Character.UnicodeScript.MYANMAR, Character.UnicodeScript.TAI_VIET, Character.UnicodeScript.MEETEI_MAYEK, Character.UnicodeScript.ETHIOPIC, Character.UnicodeScript.MEETEI_MAYEK, Character.UnicodeScript.HANGUL, Character.UnicodeScript.UNKNOWN, Character.UnicodeScript.HAN, Character.UnicodeScript.LATIN, Character.UnicodeScript.ARMENIAN, Character.UnicodeScript.HEBREW, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.LATIN, Character.UnicodeScript.COMMON, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.HANGUL, Character.UnicodeScript.COMMON, Character.UnicodeScript.LINEAR_B, Character.UnicodeScript.COMMON, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.LYCIAN, Character.UnicodeScript.CARIAN, Character.UnicodeScript.OLD_ITALIC, Character.UnicodeScript.GOTHIC, Character.UnicodeScript.UGARITIC, Character.UnicodeScript.OLD_PERSIAN, Character.UnicodeScript.DESERET, Character.UnicodeScript.SHAVIAN, Character.UnicodeScript.OSMANYA, Character.UnicodeScript.CYPRIOT, Character.UnicodeScript.IMPERIAL_ARAMAIC, Character.UnicodeScript.PHOENICIAN, Character.UnicodeScript.LYDIAN, Character.UnicodeScript.MEROITIC_HIEROGLYPHS, Character.UnicodeScript.MEROITIC_CURSIVE, Character.UnicodeScript.KHAROSHTHI, Character.UnicodeScript.OLD_SOUTH_ARABIAN, Character.UnicodeScript.AVESTAN, Character.UnicodeScript.INSCRIPTIONAL_PARTHIAN, Character.UnicodeScript.INSCRIPTIONAL_PAHLAVI, Character.UnicodeScript.OLD_TURKIC, Character.UnicodeScript.ARABIC, Character.UnicodeScript.BRAHMI, Character.UnicodeScript.KAITHI, Character.UnicodeScript.SORA_SOMPENG, Character.UnicodeScript.CHAKMA, Character.UnicodeScript.SHARADA, Character.UnicodeScript.TAKRI, Character.UnicodeScript.CUNEIFORM, Character.UnicodeScript.EGYPTIAN_HIEROGLYPHS, Character.UnicodeScript.BAMUM, Character.UnicodeScript.MIAO, Character.UnicodeScript.KATAKANA, Character.UnicodeScript.HIRAGANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.COMMON, Character.UnicodeScript.GREEK, Character.UnicodeScript.COMMON, Character.UnicodeScript.ARABIC, Character.UnicodeScript.COMMON, Character.UnicodeScript.HIRAGANA, Character.UnicodeScript.COMMON, Character.UnicodeScript.HAN, Character.UnicodeScript.COMMON, Character.UnicodeScript.INHERITED, Character.UnicodeScript.UNKNOWN };
            (Character.UnicodeScript.aliases = new HashMap(128)).put("ARAB", Character.UnicodeScript.ARABIC);
            Character.UnicodeScript.aliases.put("ARMI", Character.UnicodeScript.IMPERIAL_ARAMAIC);
            Character.UnicodeScript.aliases.put("ARMN", Character.UnicodeScript.ARMENIAN);
            Character.UnicodeScript.aliases.put("AVST", Character.UnicodeScript.AVESTAN);
            Character.UnicodeScript.aliases.put("BALI", Character.UnicodeScript.BALINESE);
            Character.UnicodeScript.aliases.put("BAMU", Character.UnicodeScript.BAMUM);
            Character.UnicodeScript.aliases.put("BATK", Character.UnicodeScript.BATAK);
            Character.UnicodeScript.aliases.put("BENG", Character.UnicodeScript.BENGALI);
            Character.UnicodeScript.aliases.put("BOPO", Character.UnicodeScript.BOPOMOFO);
            Character.UnicodeScript.aliases.put("BRAI", Character.UnicodeScript.BRAILLE);
            Character.UnicodeScript.aliases.put("BRAH", Character.UnicodeScript.BRAHMI);
            Character.UnicodeScript.aliases.put("BUGI", Character.UnicodeScript.BUGINESE);
            Character.UnicodeScript.aliases.put("BUHD", Character.UnicodeScript.BUHID);
            Character.UnicodeScript.aliases.put("CAKM", Character.UnicodeScript.CHAKMA);
            Character.UnicodeScript.aliases.put("CANS", Character.UnicodeScript.CANADIAN_ABORIGINAL);
            Character.UnicodeScript.aliases.put("CARI", Character.UnicodeScript.CARIAN);
            Character.UnicodeScript.aliases.put("CHAM", Character.UnicodeScript.CHAM);
            Character.UnicodeScript.aliases.put("CHER", Character.UnicodeScript.CHEROKEE);
            Character.UnicodeScript.aliases.put("COPT", Character.UnicodeScript.COPTIC);
            Character.UnicodeScript.aliases.put("CPRT", Character.UnicodeScript.CYPRIOT);
            Character.UnicodeScript.aliases.put("CYRL", Character.UnicodeScript.CYRILLIC);
            Character.UnicodeScript.aliases.put("DEVA", Character.UnicodeScript.DEVANAGARI);
            Character.UnicodeScript.aliases.put("DSRT", Character.UnicodeScript.DESERET);
            Character.UnicodeScript.aliases.put("EGYP", Character.UnicodeScript.EGYPTIAN_HIEROGLYPHS);
            Character.UnicodeScript.aliases.put("ETHI", Character.UnicodeScript.ETHIOPIC);
            Character.UnicodeScript.aliases.put("GEOR", Character.UnicodeScript.GEORGIAN);
            Character.UnicodeScript.aliases.put("GLAG", Character.UnicodeScript.GLAGOLITIC);
            Character.UnicodeScript.aliases.put("GOTH", Character.UnicodeScript.GOTHIC);
            Character.UnicodeScript.aliases.put("GREK", Character.UnicodeScript.GREEK);
            Character.UnicodeScript.aliases.put("GUJR", Character.UnicodeScript.GUJARATI);
            Character.UnicodeScript.aliases.put("GURU", Character.UnicodeScript.GURMUKHI);
            Character.UnicodeScript.aliases.put("HANG", Character.UnicodeScript.HANGUL);
            Character.UnicodeScript.aliases.put("HANI", Character.UnicodeScript.HAN);
            Character.UnicodeScript.aliases.put("HANO", Character.UnicodeScript.HANUNOO);
            Character.UnicodeScript.aliases.put("HEBR", Character.UnicodeScript.HEBREW);
            Character.UnicodeScript.aliases.put("HIRA", Character.UnicodeScript.HIRAGANA);
            Character.UnicodeScript.aliases.put("ITAL", Character.UnicodeScript.OLD_ITALIC);
            Character.UnicodeScript.aliases.put("JAVA", Character.UnicodeScript.JAVANESE);
            Character.UnicodeScript.aliases.put("KALI", Character.UnicodeScript.KAYAH_LI);
            Character.UnicodeScript.aliases.put("KANA", Character.UnicodeScript.KATAKANA);
            Character.UnicodeScript.aliases.put("KHAR", Character.UnicodeScript.KHAROSHTHI);
            Character.UnicodeScript.aliases.put("KHMR", Character.UnicodeScript.KHMER);
            Character.UnicodeScript.aliases.put("KNDA", Character.UnicodeScript.KANNADA);
            Character.UnicodeScript.aliases.put("KTHI", Character.UnicodeScript.KAITHI);
            Character.UnicodeScript.aliases.put("LANA", Character.UnicodeScript.TAI_THAM);
            Character.UnicodeScript.aliases.put("LAOO", Character.UnicodeScript.LAO);
            Character.UnicodeScript.aliases.put("LATN", Character.UnicodeScript.LATIN);
            Character.UnicodeScript.aliases.put("LEPC", Character.UnicodeScript.LEPCHA);
            Character.UnicodeScript.aliases.put("LIMB", Character.UnicodeScript.LIMBU);
            Character.UnicodeScript.aliases.put("LINB", Character.UnicodeScript.LINEAR_B);
            Character.UnicodeScript.aliases.put("LISU", Character.UnicodeScript.LISU);
            Character.UnicodeScript.aliases.put("LYCI", Character.UnicodeScript.LYCIAN);
            Character.UnicodeScript.aliases.put("LYDI", Character.UnicodeScript.LYDIAN);
            Character.UnicodeScript.aliases.put("MAND", Character.UnicodeScript.MANDAIC);
            Character.UnicodeScript.aliases.put("MERC", Character.UnicodeScript.MEROITIC_CURSIVE);
            Character.UnicodeScript.aliases.put("MERO", Character.UnicodeScript.MEROITIC_HIEROGLYPHS);
            Character.UnicodeScript.aliases.put("MLYM", Character.UnicodeScript.MALAYALAM);
            Character.UnicodeScript.aliases.put("MONG", Character.UnicodeScript.MONGOLIAN);
            Character.UnicodeScript.aliases.put("MTEI", Character.UnicodeScript.MEETEI_MAYEK);
            Character.UnicodeScript.aliases.put("MYMR", Character.UnicodeScript.MYANMAR);
            Character.UnicodeScript.aliases.put("NKOO", Character.UnicodeScript.NKO);
            Character.UnicodeScript.aliases.put("OGAM", Character.UnicodeScript.OGHAM);
            Character.UnicodeScript.aliases.put("OLCK", Character.UnicodeScript.OL_CHIKI);
            Character.UnicodeScript.aliases.put("ORKH", Character.UnicodeScript.OLD_TURKIC);
            Character.UnicodeScript.aliases.put("ORYA", Character.UnicodeScript.ORIYA);
            Character.UnicodeScript.aliases.put("OSMA", Character.UnicodeScript.OSMANYA);
            Character.UnicodeScript.aliases.put("PHAG", Character.UnicodeScript.PHAGS_PA);
            Character.UnicodeScript.aliases.put("PLRD", Character.UnicodeScript.MIAO);
            Character.UnicodeScript.aliases.put("PHLI", Character.UnicodeScript.INSCRIPTIONAL_PAHLAVI);
            Character.UnicodeScript.aliases.put("PHNX", Character.UnicodeScript.PHOENICIAN);
            Character.UnicodeScript.aliases.put("PRTI", Character.UnicodeScript.INSCRIPTIONAL_PARTHIAN);
            Character.UnicodeScript.aliases.put("RJNG", Character.UnicodeScript.REJANG);
            Character.UnicodeScript.aliases.put("RUNR", Character.UnicodeScript.RUNIC);
            Character.UnicodeScript.aliases.put("SAMR", Character.UnicodeScript.SAMARITAN);
            Character.UnicodeScript.aliases.put("SARB", Character.UnicodeScript.OLD_SOUTH_ARABIAN);
            Character.UnicodeScript.aliases.put("SAUR", Character.UnicodeScript.SAURASHTRA);
            Character.UnicodeScript.aliases.put("SHAW", Character.UnicodeScript.SHAVIAN);
            Character.UnicodeScript.aliases.put("SHRD", Character.UnicodeScript.SHARADA);
            Character.UnicodeScript.aliases.put("SINH", Character.UnicodeScript.SINHALA);
            Character.UnicodeScript.aliases.put("SORA", Character.UnicodeScript.SORA_SOMPENG);
            Character.UnicodeScript.aliases.put("SUND", Character.UnicodeScript.SUNDANESE);
            Character.UnicodeScript.aliases.put("SYLO", Character.UnicodeScript.SYLOTI_NAGRI);
            Character.UnicodeScript.aliases.put("SYRC", Character.UnicodeScript.SYRIAC);
            Character.UnicodeScript.aliases.put("TAGB", Character.UnicodeScript.TAGBANWA);
            Character.UnicodeScript.aliases.put("TALE", Character.UnicodeScript.TAI_LE);
            Character.UnicodeScript.aliases.put("TAKR", Character.UnicodeScript.TAKRI);
            Character.UnicodeScript.aliases.put("TALU", Character.UnicodeScript.NEW_TAI_LUE);
            Character.UnicodeScript.aliases.put("TAML", Character.UnicodeScript.TAMIL);
            Character.UnicodeScript.aliases.put("TAVT", Character.UnicodeScript.TAI_VIET);
            Character.UnicodeScript.aliases.put("TELU", Character.UnicodeScript.TELUGU);
            Character.UnicodeScript.aliases.put("TFNG", Character.UnicodeScript.TIFINAGH);
            Character.UnicodeScript.aliases.put("TGLG", Character.UnicodeScript.TAGALOG);
            Character.UnicodeScript.aliases.put("THAA", Character.UnicodeScript.THAANA);
            Character.UnicodeScript.aliases.put("THAI", Character.UnicodeScript.THAI);
            Character.UnicodeScript.aliases.put("TIBT", Character.UnicodeScript.TIBETAN);
            Character.UnicodeScript.aliases.put("UGAR", Character.UnicodeScript.UGARITIC);
            Character.UnicodeScript.aliases.put("VAII", Character.UnicodeScript.VAI);
            Character.UnicodeScript.aliases.put("XPEO", Character.UnicodeScript.OLD_PERSIAN);
            Character.UnicodeScript.aliases.put("XSUX", Character.UnicodeScript.CUNEIFORM);
            Character.UnicodeScript.aliases.put("YIII", Character.UnicodeScript.YI);
            Character.UnicodeScript.aliases.put("ZINH", Character.UnicodeScript.INHERITED);
            Character.UnicodeScript.aliases.put("ZYYY", Character.UnicodeScript.COMMON);
            Character.UnicodeScript.aliases.put("ZZZZ", Character.UnicodeScript.UNKNOWN);
        }
    }
    
    private static class CharacterCache
    {
        static final Character[] cache;
        
        static {
            cache = new Character[128];
            for (int i = 0; i < Character.CharacterCache.cache.length; ++i) {
                Character.CharacterCache.cache[i] = new Character((char)i);
            }
        }
    }
}
